#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"

#include "misc.h"

#include "acc_entry.h"

/* this function is needed by GCompletion */
static gchar *acc_entry_get_account_name(AccountData *account)
{
  return account->name;
}

/* this function is responsible for completion in case PgDn is pressed */
static gint acc_entry_key_press_event(GtkWidget *entry, 
				      GdkEventKey *key_event, 
				      gpointer callback_data)
{
  GCompletion *completion;
  gchar *new_prefix;

  g_return_val_if_fail(entry != NULL, FALSE);
  g_return_val_if_fail(key_event != NULL, FALSE);

  if (key_event->keyval != GDK_Page_Down)
    return FALSE;

  completion = g_completion_new((GCompletionFunc) acc_entry_get_account_name);
  g_completion_add_items(completion, acc_list);
  g_completion_complete(completion, 
			gtk_entry_get_text(GTK_ENTRY(entry)), 
			&new_prefix);

  gtk_entry_set_text(GTK_ENTRY(entry), new_prefix);
  
  g_free(new_prefix);
  g_completion_free(completion);

  return TRUE;
}

static void acc_entry_selected_from_tree(GtkWidget *ctree, 
					 GtkCTreeNode *node, 
					 gint column, 
					 GnomeUcombo *acc_entry)
{
  AccountData *account;

  account = AccountPtr(gtk_ctree_node_get_row_data(GTK_CTREE(ctree), node));
  if (account != NULL)
    gtk_entry_set_text(GTK_ENTRY(acc_entry->entry), (gchar *) account->name);
  else
    gtk_entry_set_text(GTK_ENTRY(acc_entry->entry), _("<none>"));

  gnome_ucombo_hide_popup(acc_entry);
}

GtkWidget *acc_entry_new(const gboolean with_none)
{
	GtkWidget *acc_entry;

	GtkWidget *ctree;

	ctree = misc_build_ctree(acc_list, with_none ? _("<none>") : NULL);
	acc_entry = gnome_ucombo_new(ctree);
	gtk_signal_connect(GTK_OBJECT(GNOME_UCOMBO(acc_entry)->entry), 
			   "key_press_event", 
			   GTK_SIGNAL_FUNC(acc_entry_key_press_event), NULL);

	gtk_signal_connect(GTK_OBJECT(ctree), "tree_select_row", 
			   acc_entry_selected_from_tree, acc_entry);

	return acc_entry;
}

gint acc_entry_get_account(GtkWidget *acc_entry, AccountData **ret)
{
  GList *cursor;
  gchar *text;

  g_assert(GNOME_IS_UCOMBO(acc_entry));
  
  text = gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(acc_entry)->entry));
  if (strcmp(text, _("<none>")) == 0) {
    /* the caller have to check if he likes the null value returned */
    *ret = NULL; 
    return 0;
  }

  for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
    if (!strcmp(text, AccountPtr(cursor->data)->name)) {
      *ret = AccountPtr(cursor->data);
      return 0;
    }

  return 1; /* no matching account found */
}
