
#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"
#include "acc_entry.h"

#include "data.h"
#include "interface.h"

#include "account_dlg.h"

struct _AccountEdit {
     GtkWidget *dialog;

     GtkWidget *name;
     GtkWidget *parent;
     GtkWidget *unlimited;
     GtkWidget *reversed;

     gint error; /* 0 - ok, 1-4 - number of entry which is wrong */
} account_dlg;

void account_dlg_build_dialog(const gchar *title)
{
    account_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(account_dlg.dialog), GTK_WINDOW(interface.application));

    {
	GtkWidget *table;
	table = gtk_table_new(2, 3, FALSE);

	/* insert entry widgets */
	account_dlg.parent = acc_entry_new(TRUE);
	gtk_widget_show(account_dlg.parent);
	gtk_table_attach(GTK_TABLE(table), account_dlg.parent, 1, 2, 1, 2, GTK_FILL, 0, 3, 3);

	account_dlg.name = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(account_dlg.name), ACCOUNT_NAME_SIZE-1);
	gtk_widget_show(account_dlg.name);
	gtk_table_attach(GTK_TABLE(table), account_dlg.name, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 3, 3);

	/* insert check buttons */
	{
	     GtkWidget *hbox;
	     hbox = gtk_hbox_new(TRUE, 0);
	     gtk_widget_show(hbox);

	     account_dlg.unlimited = gtk_check_button_new_with_label(_("Unlimited"));
	     gtk_widget_show(account_dlg.unlimited);
	     gtk_box_pack_start(GTK_BOX(hbox), account_dlg.unlimited, TRUE, FALSE, 0);
	     
	     account_dlg.reversed = gtk_check_button_new_with_label(_("Reversed"));
	     gtk_widget_show(account_dlg.reversed);
	     gtk_box_pack_start(GTK_BOX(hbox), account_dlg.reversed, TRUE, FALSE, 0);

	     gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 2, 3, GTK_FILL, 0, 3, 3);
	}

	/* insert labels */
	{
	    GtkWidget *label;

	    label = gtk_label_new(_("Name:"));
	    gtk_widget_show(label);
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Parent:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);
	}

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(account_dlg.dialog)->vbox), table, TRUE, TRUE, 0);
	gtk_widget_show(table);
    }
}

void account_dlg_preset_values(const AccountData *account)
{
     gtk_entry_set_text(GTK_ENTRY(account_dlg.name), account->name);

     if (account->parent == NULL)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(account_dlg.parent)->entry), _("<none>"));
     else
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(account_dlg.parent)->entry), account->parent->name);

     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(account_dlg.unlimited), account->unlimited);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(account_dlg.reversed), account->reversed);
}

void account_dlg_handle_errors(void)
{
     switch (account_dlg.error) {
     case 1: /* name */
	  gtk_widget_grab_focus(account_dlg.name);
	  gdk_beep();
	  break;
     case 2: /* parent */
	  gtk_widget_grab_focus(account_dlg.parent);
	  gdk_beep();
	  break;
     default:
	  g_assert_not_reached();
     }
}

/* returns 0 on success or any other value on failure */
void account_dlg_main(AccountData *account)
{
     GList *cursor;

     gchar *name_val = gtk_entry_get_text(GTK_ENTRY(account_dlg.name));
     AccountData *parent_val;

     { /* check if parent is alright */
	  AccountData *check;

	  if (acc_entry_get_account(account_dlg.parent, &parent_val)) {
	       account_dlg.error = 2;
	       return;
	  }

	  check = parent_val;
	  while (check != NULL) {
	       if (check == account) {
		    account_dlg.error = 2;
		    return;
	       }
	       check = check->parent;
	  }
     }

     /* check if the name is unique */
     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
	  if (cursor->data != account)
	       if (!strcmp(AccountPtr(cursor->data)->name, name_val)) {
		    g_print("just checking - the name isn't unique?\n");
		    account_dlg.error = 1;
		    return;
	       }

     /* and now we can actually change the account */
     strcpy(account->name, name_val);
     account->parent = parent_val;
     account->unlimited = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(account_dlg.unlimited));
     account->reversed = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(account_dlg.reversed));

     account_dlg.error = 0;
}

gboolean account_dlg_execute(const gchar *title, AccountData *account)
{
    account_dlg_build_dialog(title);
    account_dlg_preset_values(account);

    while (TRUE) {
	int ret;

	ret = gnome_dialog_run(GNOME_DIALOG(account_dlg.dialog));
	switch (ret) {
	case 0:		/* OK */
	    account_dlg_main(account);
	    if (account_dlg.error == 0) {
		gnome_dialog_close(GNOME_DIALOG(account_dlg.dialog));
		return TRUE;
	    } else
		 account_dlg_handle_errors();

	    break;
	case 1:		/* Cancel */
	    gnome_dialog_close(GNOME_DIALOG(account_dlg.dialog));
	case -1:		/* Esc or "closed window" */
	    return FALSE;
	}
    };
}
