
#include <config.h>

#include <gnome.h>

#include "interface.h"

#include "py_module.h"
#include "data.h"
#include "schedule.h"
#include "report.h"
#include "report_druid.h"

#include "account_dlg.h"
#include "transaction_dlg.h"
#include "schedule_dlg.h"
#include "limit_dlg.h"
#include "prefs_dlg.h"

#include "misc.h"

#include "actions.h"

/* returns 0 - yes, 1 - no, 2 - cancel */
gint action_do_you_want_to_save(void)
{
     static gboolean action_do_you_want_to_save_is_active = FALSE;

     if (action_do_you_want_to_save_is_active)
	  return 2; /* cancel (this question is just being asked) */

    if (interface.changed) {
	 gint return_val;
	 GtkWidget *msgbox;
	 msgbox = gnome_message_box_new(_("Data has been modified. Do you wish to save it?"), 
					GNOME_MESSAGE_BOX_QUESTION,
					GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
					GNOME_STOCK_BUTTON_CANCEL, NULL);

	action_do_you_want_to_save_is_active = TRUE;
	return_val = gnome_dialog_run_and_close(GNOME_DIALOG(msgbox));
	action_do_you_want_to_save_is_active = FALSE;
	return return_val;
    } else
	return 1;		/* not changed - don't want to save */
}

void action_error_msg(const gchar *normal, const gchar *fixed)
{
     /* FIXME - I don't know if ref/unref should be done for gdkfont and gtkstyle */

     GtkWidget *msgbox;
     GtkWidget *label;
     GtkStyle *style;

     msgbox = gnome_dialog_new(_("Error"), GNOME_STOCK_BUTTON_OK, NULL);

     if (normal != NULL) {
	  label = gtk_label_new(normal);
	  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	  gtk_widget_show(label);
	  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(msgbox)->vbox), label, TRUE, TRUE, 0);
     }

     if (fixed != NULL) {
	  label = gtk_label_new(fixed);
	  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
	  
	  style = gtk_style_copy(gtk_widget_get_style(label));
	  style->font = gdk_font_load("fixed");
	  gtk_widget_set_style(label, style);

	  gtk_widget_show(label);
	  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(msgbox)->vbox), label, TRUE, TRUE, 0);
     }

     gnome_dialog_run_and_close(GNOME_DIALOG(msgbox));
}

/********************************
 *      FILE MENU               *
 ********************************/

void action_new(GtkWidget * widget, gpointer callback_data)
{
    switch (action_do_you_want_to_save()) {
    case 0:			/* yes */
	action_save(widget, action_new);
	break;
    case 1:			/* 'no' or no need */
	data_done();
	data_init();

	interface_setup_filename("");
	gtk_clist_clear(GTK_CLIST(interface.accounts_tree));
	gtk_clist_clear(GTK_CLIST(interface.transactions_registry));
	interface.changed = FALSE;

	{
	     GtkWidget *msgbox;
	     msgbox = gnome_message_box_new(_("Do you want me to set up default accounts ?"), 
					    GNOME_MESSAGE_BOX_QUESTION,
					    GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO, NULL);
	     if (gnome_dialog_run_and_close(GNOME_DIALOG(msgbox)) == 0) {
		 data_set_standard_accounts();
		 interface_build_all_accounts();
	     }
	}

	break;
    case 2:			/* cancel */
	break;
    default:			
	g_assert_not_reached();
    }
}

void action_open_direct(const gchar *filename)
{
     interface_setup_filename(filename);
     interface.changed = FALSE;

     data_done();
     data_init();
     data_read(filename);
     data_recalculate_accounts();

     interface_build_all_accounts();
     gtk_ctree_expand_to_depth(GTK_CTREE(interface.accounts_tree), NULL,
			       gnome_config_get_int("/emma/Open/Expand Accounts To Depth=1"));

     interface_build_all_transactions();
     action_limit_apply(NULL, GINT_TO_POINTER(TRUE));

     interface_build_all_schedules();
     process_scheduled_transactions();
}

void action_open_execute(GtkWidget * widget, GtkWidget * file_selection)
{
     action_open_direct(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
     gtk_widget_destroy(file_selection);
}

void action_open(GtkWidget * widget, gpointer callback_data)
{
    switch (action_do_you_want_to_save()) {
    case 0:			/* yes */
	action_save(widget, action_open);
	break;
    case 1:
	{			/* 'no' or no need */
	    GtkWidget *file_selection;

	    file_selection = gtk_file_selection_new(_("Open..."));

	    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(file_selection)->cancel_button), "clicked",
				      (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(file_selection));

	    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selection)->ok_button), "clicked",
		    (GtkSignalFunc) action_open_execute, file_selection);

	    gtk_widget_show(file_selection);

	    break;
	}
    case 2:			/* cancel */
	 break;
    default:		       
	 g_assert_not_reached();
    }
}

void action_save(GtkWidget * widget, gpointer call_me_later)
{
    if (strcmp(interface.filename, "") == 0)
	action_saveas(widget, call_me_later);
    else {
	data_write(interface.filename);
	interface.changed = FALSE;

	if (call_me_later != NULL)
	     /* FIXME - is this really working ? */
	     GTK_SIGNAL_FUNC(call_me_later) (widget, NULL);
    }
}

void action_saveas_execute(GtkWidget * widget, GtkWidget * file_selection)
{
     if (!g_file_exists(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)))) {
	  data_write(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
	  interface_setup_filename(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
	  gtk_widget_destroy(file_selection);
	  interface.changed = FALSE;
     } else {			/* file exists - overwrite ? */
	  GtkWidget *msgbox;
	  char msg[1024];
	  int ret;

	  sprintf(msg, _("Do you wish to overwrite %s ?"),
		  g_basename(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection))));
	  
	  msgbox = gnome_message_box_new(msg, GNOME_MESSAGE_BOX_QUESTION,
					 GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
					 GNOME_STOCK_BUTTON_CANCEL, NULL);

	  ret = gnome_dialog_run_and_close(GNOME_DIALOG(msgbox));
	  switch (ret) {
	  case 0:		/* yes */
	       data_write(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
	       interface_setup_filename(gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selection)));
	       interface.changed = FALSE;
	  case 2:		/* cancel */
	       gtk_widget_destroy(file_selection);
	       break;
	  case 1:		/* no */
	       break;
	  default:	       
	       g_assert_not_reached();
	  }			
     }
}

void action_saveas(GtkWidget * widget, gpointer call_me_later)
{
    GtkWidget *file_selection;

    file_selection = gtk_file_selection_new(_("Save as..."));

    gtk_signal_connect_object(GTK_OBJECT(GTK_FILE_SELECTION(file_selection)->cancel_button), "clicked",
	 (GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT(file_selection));

    gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selection)->ok_button), "clicked",
		  (GtkSignalFunc) action_saveas_execute, file_selection);

    if (call_me_later != NULL)
	gtk_signal_connect(GTK_OBJECT(file_selection), "destroy",
			   (GtkSignalFunc) call_me_later, file_selection);

    gtk_widget_show(file_selection);
}

void action_preferences(GtkWidget * widget, gpointer callback_data)
{
     if (prefs_dlg_execute()) {
	  /* OK */
     }
}

void action_quit(GtkWidget * widget, gpointer callback_data)
{
    GdkEventAny event;
    gint return_val;

    gtk_signal_emit_by_name(GTK_OBJECT(interface.application), "delete_event", &event, &return_val);
}

/********************************
 *         ACCOUNTS             *
 ********************************/

void action_account_new(GtkWidget * widget, gpointer callback_data)
{
     AccountData *account;

     account = g_new(AccountData, 1);
     strcpy(account->name, "");
     account->parent = NULL;
     account->unlimited = FALSE;
     account->reversed = FALSE;
     account->amount = 0;
     account->tree_node = NULL;
     
     if (account_dlg_execute(_("Account new..."), account)) {
	  interface.changed = TRUE;
	  acc_list = g_list_append(acc_list, account);
	  interface_build_account_node(account);
	  if (account->parent == NULL)
	       gtk_ctree_sort_node(GTK_CTREE(interface.accounts_tree), NULL);
	  else
	       gtk_ctree_sort_node(GTK_CTREE(interface.accounts_tree), account->parent->tree_node);

	  /* expand node's parents, so it is visible */
	  account = account->parent;
	  while (account != NULL) {
	       gtk_ctree_expand(GTK_CTREE(interface.accounts_tree), account->tree_node);
	       account = account->parent;
	  }
     } else {
	  g_free(account);
     }
}

void action_account_edit(GtkWidget * widget, gpointer callback_data)
{
     AccountData *account;

     if (GTK_CLIST(interface.accounts_tree)->selection == NULL)
	  return;

     account = gtk_ctree_node_get_row_data(GTK_CTREE(interface.accounts_tree), 
					   GTK_CLIST(interface.accounts_tree)->selection->data);

     if (account_dlg_execute(_("Account edit..."), account)) {
	  interface_setup_account_node(account);
	  data_recalculate_accounts();
	  interface_setup_all_accounts_amount();
	  interface.changed = TRUE;
     }
}

void action_account_delete(GtkWidget * widget, gpointer callback_data)
{
     GList *cursor;
     AccountData *account;

     g_return_if_fail(GTK_CLIST(interface.accounts_tree)->selection != NULL);

     account = gtk_ctree_node_get_row_data(GTK_CTREE(interface.accounts_tree), 
					   GTK_CLIST(interface.accounts_tree)->selection->data);

     /* check if there are transactions associated with the account to be removed */
     for (cursor = trans_list; cursor != NULL; cursor = g_list_next(cursor))
	  if ((TransactionPtr(cursor->data)->acc_from == account) ||
	      (TransactionPtr(cursor->data)->acc_to == account)) {
	       /* FIXME - something is very wrong here? */
	       gnome_ok_dialog(_("There are some transactions associated with this account.\nCan't proceed!"));
	       return;
	  }

     /* take care of children of the account to be removed */
     gtk_clist_freeze(GTK_CLIST(interface.accounts_tree));
     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
	  if (AccountPtr(cursor->data)->parent == account) {
	       AccountPtr(cursor->data)->parent = account->parent;
	       interface_setup_account_node(AccountPtr(cursor->data));
	  }

     /* remove the account */
     acc_list = g_list_remove(acc_list, account);
     gtk_ctree_remove_node(GTK_CTREE(interface.accounts_tree), account->tree_node);
     g_free(account);

     interface.changed = TRUE;
     gtk_clist_thaw(GTK_CLIST(interface.accounts_tree));
}

/********************************
 *      TRANSACTIONS            *
 ********************************/

void action_transaction_new(GtkWidget * widget, gpointer initial_from_acc)
{
     TransactionData *transaction;
     GList *trans_new_list = NULL;

     static TransactionData def_trans;
     static gboolean first_time = TRUE;

     if (first_time) {
	  def_trans.amount = 0;
	  def_trans.acc_to = NULL;
	  g_date_set_current(&(def_trans.date));
	  strcpy(def_trans.desc, "");
	  def_trans.inbound = TRUE;

	  first_time = FALSE;
     }

     if (initial_from_acc != NULL)
	  def_trans.acc_from = AccountPtr(initial_from_acc);

     transaction = g_new(TransactionData, 1);
     *transaction = def_trans;
     trans_new_list = g_list_prepend(trans_new_list, transaction);
     
     if (transaction_dlg_execute(_("New transaction ..."), trans_new_list)) {
	  trans_list = g_list_append(trans_list, transaction);
	  data_recalculate_accounts();
	  interface_setup_all_accounts_amount();
	  interface_build_transaction(transaction);
	  interface_setup_transaction_colors(transaction);
	  interface.changed = TRUE;

	  def_trans.acc_from = transaction->acc_from;
	  def_trans.acc_to = transaction->acc_to;
	  def_trans.date = transaction->date;
     } else
	  g_free(transaction);
     
     g_list_free(trans_new_list);
}

void action_transaction_edit(GtkWidget * widget, gpointer callback_data)
{
    GList *selection, *trans_selection = NULL;

    if (GTK_CLIST(interface.transactions_registry)->selection == NULL)
	    return;

    /* make up a list of transactions to edit */
    for (selection = GTK_CLIST(interface.transactions_registry)->selection; 
	 selection != NULL; selection = g_list_next(selection))
	trans_selection = g_list_prepend(trans_selection,
					 gtk_clist_get_row_data(GTK_CLIST(interface.transactions_registry),
								GPOINTER_TO_INT(selection->data)));

    if (transaction_dlg_execute(_("Transaction edit..."), trans_selection)) {
	 GList *cursor;
	 
	 gtk_clist_freeze(GTK_CLIST(interface.transactions_registry));
	 for (cursor = trans_selection; cursor != NULL; cursor = g_list_next(cursor))
	      interface_setup_transaction(TransactionPtr(cursor->data));

	 data_recalculate_accounts();
	 interface_setup_all_accounts_amount();
	 interface.changed = TRUE;

	 gtk_clist_thaw(GTK_CLIST(interface.transactions_registry));
    }

    g_list_free(trans_selection);
}

void action_transaction_delete(GtkWidget * widget, gpointer callback_data)
{
    GList *selection;
    gint row_to_remove;

    if (GTK_CLIST(interface.transactions_registry)->selection == NULL)
	    return;

    /* this whole function is a hack, not very ugly, but a hack */

    gtk_clist_freeze(GTK_CLIST(interface.transactions_registry));

    for (selection = GTK_CLIST(interface.transactions_registry)->selection; 
	 selection != NULL; selection = g_list_next(selection)) {
	TransactionData *trans = TransactionPtr(gtk_clist_get_row_data(GTK_CLIST(interface.transactions_registry),
								       GPOINTER_TO_INT(selection->data)));
	gtk_clist_set_row_data(GTK_CLIST(interface.transactions_registry), trans->row, NULL);
	trans_list = g_list_remove(trans_list, trans);
	g_free(trans);
    }

    for (row_to_remove = 0; row_to_remove < GTK_CLIST(interface.transactions_registry)->rows; row_to_remove++) 
	 if (gtk_clist_get_row_data(GTK_CLIST(interface.transactions_registry), row_to_remove) == NULL) {
	      gtk_clist_remove(GTK_CLIST(interface.transactions_registry), row_to_remove);
	      row_to_remove--;
	 }

    interface_setup_all_transactions_colors();

    gtk_clist_thaw(GTK_CLIST(interface.transactions_registry));

    data_recalculate_accounts();
    interface_setup_all_accounts_amount();
    interface.changed = TRUE;
}

void transaction_selection_eval(const gchar *expr, const gboolean select)
{
     GList *cursor;

     py_module_compile_limit(expr);
     if (py_module_error_occurred())
	  return;

     for (cursor = trans_list; cursor !=NULL ; cursor = g_list_next(cursor)) {
	  TransactionData *trans = TransactionPtr(cursor->data);
	  gboolean result;

	  if (!trans->inbound)
	       continue;

	  result = py_module_eval_limit(trans);
	  if (py_module_error_occurred())
	       break;

	  if (result) {
	       if (select)
		    gtk_clist_select_row(GTK_CLIST(interface.transactions_registry), trans->row, 0);
	       else
		    gtk_clist_unselect_row(GTK_CLIST(interface.transactions_registry), trans->row, 0);
	  }
     }

     py_module_release_compiled_limit();
}

void transaction_selection_main(const gchar *title, const gboolean select)
{
     gchar expr_text[PY_EXPRESSION_MAX_SIZE] = "";

     while (limit_dlg_execute(title, expr_text)) {
	  transaction_selection_eval(expr_text, select);
	  if (py_module_error_occurred()) {
	       action_error_msg(_("Error in expression!\n\nPython says:\n"), 
				py_module_get_error_string());
	       py_module_clear_error();
	  } else 
	       break;
     }
}

void action_transaction_select(GtkWidget * widget, gpointer callback_data)
{
     transaction_selection_main(_("Select transactions..."), TRUE);
}

void action_transaction_unselect(GtkWidget * widget, gpointer callback_data)
{
     transaction_selection_main(_("Unselect transactions..."), FALSE);
}

void action_transaction_reverse_selection(GtkWidget * widget, gpointer callback_data)
{
    GList *selection, *selected_trans = NULL;

    gtk_clist_freeze(GTK_CLIST(interface.transactions_registry));

    /* make up a list of already selected transactions */
    for (selection = GTK_CLIST(interface.transactions_registry)->selection; 
	 selection != NULL; selection = g_list_next(selection))
	selected_trans = g_list_prepend(selected_trans, selection->data);

    gtk_clist_select_all(GTK_CLIST(interface.transactions_registry));

    /* unselect previously selected transactions */
    for (selection = selected_trans; selection != NULL; selection = g_list_next(selection))
	 gtk_clist_unselect_row(GTK_CLIST(interface.transactions_registry), GPOINTER_TO_INT(selection->data), 0);

    g_list_free(selected_trans);
    gtk_clist_thaw(GTK_CLIST(interface.transactions_registry));
}

void action_transaction_info(GtkWidget * widget, gpointer callback_data)
{
    GList *selection;
    gint count = 0;
    gdouble avg,sum = 0;

    gchar msg[1000]; /* FIXME: Eeergh - another _fixed length_ string :-( */
    GtkWidget *msgbox;

    if (GTK_CLIST(interface.transactions_registry)->selection == NULL)
	    return;

    /* goes through every selected transaction and calculates sum and count */
    for (selection = GTK_CLIST(interface.transactions_registry)->selection; 
	 selection != NULL; selection = g_list_next(selection)) {
	 TransactionData *transaction; 
	 transaction = TransactionPtr(gtk_clist_get_row_data(GTK_CLIST(interface.transactions_registry), 
							     GPOINTER_TO_INT(selection->data)));
	 count++;
	 sum += transaction->amount;
    }
    avg = sum / count;

    sprintf(msg, 
	  _("Selected \t= %d \n"
	    "Sum \t= %.2lf \n"
	    "Average amount \t= %.2lf \n"), 
	    count, sum, avg);
	    
    msgbox = gnome_message_box_new(msg,
				   GNOME_MESSAGE_BOX_INFO,
				   GNOME_STOCK_BUTTON_OK, NULL);

    gnome_dialog_run_and_close(GNOME_DIALOG(msgbox));
}

void action_transaction_set_highlight_mode(GtkWidget * widget, gpointer mode)
{
     interface.trans_highlight_mode = GPOINTER_TO_INT(mode);
     interface_setup_all_transactions_colors();
}

/********************************
 *      SCHEDULE                *
 ********************************/

void action_schedule_new(GtkWidget * widget, gpointer callback_data)
{
     GList *schedule_new_list = NULL;
     ScheduleData *schedule;

     schedule = g_new(ScheduleData, 1);
     schedule->amount = 0;
     schedule->acc_from = NULL;
     schedule->acc_to = NULL;
     schedule->factor = 1;
     g_date_set_current(&(schedule->date));
     strcpy(schedule->desc, "");
     schedule->frequency = 0;

     schedule_new_list = g_list_prepend(schedule_new_list, schedule);
     
     if (schedule_dlg_execute(_("New scheduled transaction ..."), schedule_new_list)) {
	  schedule_list = g_list_append(schedule_list, schedule);
	  interface_build_schedule(schedule);
	  interface.changed = TRUE;
     } else
	  g_free(schedule);
     
     g_list_free(schedule_new_list);
}

void action_schedule_edit(GtkWidget * widget, gpointer callback_data)
{
    GList *selection, *schedule_selection = NULL;

    if (GTK_CLIST(interface.schedule_list)->selection == NULL)
	    return;

    /* make up a list of schedules to edit */
    for (selection = GTK_CLIST(interface.schedule_list)->selection; 
	 selection != NULL; selection = g_list_next(selection))
	schedule_selection = g_list_prepend(schedule_selection,
					 gtk_clist_get_row_data(GTK_CLIST(interface.schedule_list),
								GPOINTER_TO_INT(selection->data)));

    if (schedule_dlg_execute(_("Schedule edit..."), schedule_selection)) {
	 GList *cursor;
	 
	 gtk_clist_freeze(GTK_CLIST(interface.schedule_list));
	 for (cursor = schedule_selection; cursor != NULL; cursor = g_list_next(cursor))
	      interface_setup_schedule(SchedulePtr(cursor->data));

	 interface.changed = TRUE;

	 gtk_clist_thaw(GTK_CLIST(interface.schedule_list));
    }

    g_list_free(schedule_selection);
}

void action_schedule_delete(GtkWidget * widget, gpointer callback_data)
{
    GList *selection;
    gint row_to_remove;

    if (GTK_CLIST(interface.schedule_list)->selection == NULL)
	    return;

    /* this whole function is a hack, not very ugly, but a hack */

    gtk_clist_freeze(GTK_CLIST(interface.schedule_list));

    for (selection = GTK_CLIST(interface.schedule_list)->selection; 
	 selection != NULL; selection = g_list_next(selection)) {
	ScheduleData *schedule = SchedulePtr(gtk_clist_get_row_data(GTK_CLIST(interface.schedule_list),
								       GPOINTER_TO_INT(selection->data)));
	gtk_clist_set_row_data(GTK_CLIST(interface.schedule_list), schedule->row, NULL);
	schedule_list = g_list_remove(schedule_list, schedule);
	g_free(schedule);
    }

    for (row_to_remove = 0; row_to_remove < GTK_CLIST(interface.schedule_list)->rows; row_to_remove++) 
	 if (gtk_clist_get_row_data(GTK_CLIST(interface.schedule_list), row_to_remove) == NULL) {
	      gtk_clist_remove(GTK_CLIST(interface.schedule_list), row_to_remove);
	      row_to_remove--;
	 }

    gtk_clist_thaw(GTK_CLIST(interface.schedule_list));

    interface.changed = TRUE;
}

/********************************
 *          REPORT              *
 ********************************/

void action_report_druid(GtkWidget * widget, gpointer callback_data)
{
  report_druid_execute();
}

void action_report_rebuild(GtkWidget * widget, gpointer callback_data)
{
     report_rebuild();
     if (py_module_error_occurred()) {
	  action_error_msg(_("Error in groups expressions!\n\nPython says:\n"), 
			   py_module_get_error_string());
	  py_module_clear_error();
     }

}

static void action_report_filter_execute(gchar *command, gpointer callback_data)
{
     if (command != NULL) {
	  report_filter(command);
	  g_free(command);
     }
}

void action_report_filter(GtkWidget * widget, gpointer command)
{
     GtkWidget *dialog;

     dialog = gnome_request_dialog(FALSE, 
				   _("Command:"), (gchar *)command, -1,
				   (GnomeStringCallback) action_report_filter_execute,
				   NULL, GTK_WINDOW(interface.application));
}
/********************************
 *          LIMIT               *
 ********************************/

void action_limit_edit(GtkWidget * widget, gpointer callback_data)
{
     gchar limit_text[PY_EXPRESSION_MAX_SIZE];
     strcpy(limit_text, interface_get_limit());

     while (limit_dlg_execute(_("Edit limit..."), limit_text)) {
	  interface_setup_limit(limit_text);
	  action_limit_apply(NULL, GINT_TO_POINTER(FALSE));

	  if (py_module_error_occurred()) {
	       action_error_msg(_("Error in limit expression!\n\nPython says:\n"), 
				py_module_get_error_string());
	       py_module_clear_error();
	  } else
	       break;
     }
}

void action_limit_clear(GtkWidget * widget, gpointer callback_data)
{
     interface_setup_limit("");
     action_limit_apply(NULL, GINT_TO_POINTER(TRUE)); /* check for errors just in case */
}

void action_limit_apply(GtkWidget * widget, gpointer callback_data)
{
     gboolean resolve_errors = GPOINTER_TO_INT(callback_data);

     data_rebound_transactions(interface_get_limit());
     data_recalculate_accounts();
     interface_setup_all_accounts_amount();
     interface_build_all_transactions();

     if (resolve_errors && py_module_error_occurred()) {
	  action_error_msg(_("Error in limit expression!\n\nPython says:\n"), 
			   py_module_get_error_string());
	  py_module_clear_error();
     }
}

/********************************
 *   not implemented yet        *
 ********************************/

void action_notimplemented(GtkWidget * widget, gpointer callback_data)
{
     gdk_beep();
     g_print("Sorry - not implemented yet\n");
}

/********************************
 *        ABOUT                 *
 ********************************/

void action_about(GtkWidget * widget, gpointer callback_data)
{
    GtkWidget *about;
    const gchar *authors[] =
    {
	"Lukasz Anforowicz <lanfor@atr.bydgoszcz.pl>",
	NULL
    };

    gchar *info_text;

    int i = 0;
    while (authors[i] != NULL)
	authors[i++] = _(authors[i]);

    info_text = g_strdup_printf(_("Emma is simple and flexible money management program "
				  "for GNOME/Gtk+ with Python embeded\n"
				  "Emma's homepage : %s"), "http://rainbow.mimuw.edu.pl/~la181249");

    about = gnome_about_new("Emma", VERSION,"(C) 1999, Lukasz Anforowicz", authors, info_text, NULL);
    gtk_widget_show(about);
}


