/*
 * Date editor widget
 * Copyright (C) 1998 the Free Software Foundation
 *
 * Author: Lukasz Anforowicz
 *         Miguel de Icaza (responsible for date-edit from which I got most of the below code)
 */

#include <config.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <libgnomeui/gnome-uidefs.h>

#include "gnome-ucombo.h"


static void gnome_ucombo_init(GnomeUcombo * ucombo);
static void gnome_ucombo_class_init(GnomeUcomboClass * class);
static void gnome_ucombo_destroy(GtkObject * object);
static void gnome_ucombo_forall(GtkContainer * container,
				gboolean include_internals,
				GtkCallback callback,
				gpointer callbabck_data);

static GtkHBoxClass *parent_class;

/**
 * gnome_ucombo_get_type:
 *
 * Returns the GtkType for the GnomeUcombo widget
 */
guint
gnome_ucombo_get_type(void)
{
    static guint ucombo_type = 0;

    if (!ucombo_type) {
	GtkTypeInfo ucombo_info =
	{
	    "GnomeUcombo",
	    sizeof(GnomeUcombo),
	    sizeof(GnomeUcomboClass),
	    (GtkClassInitFunc) gnome_ucombo_class_init,
	    (GtkObjectInitFunc) gnome_ucombo_init,
	    NULL,
	    NULL,
	};

	ucombo_type = gtk_type_unique(gtk_hbox_get_type(), &ucombo_info);
    }
    return ucombo_type;
}

void gnome_ucombo_hide_popup(GnomeUcombo * ucombo)
{
    gtk_widget_hide(ucombo->popwindow);
    gtk_grab_remove(ucombo->popwindow);
    gdk_pointer_ungrab(GDK_CURRENT_TIME);
}

static gint
 delete_popup(GtkWidget * widget, gpointer data)
{
    GnomeUcombo *ucombo;

    ucombo = data;
    gnome_ucombo_hide_popup(ucombo);

    return TRUE;
}

static gint
 key_press_popup(GtkWidget * widget, GdkEventKey * event, gpointer data)
{
    GnomeUcombo *ucombo;

    if (event->keyval != GDK_Escape)
	return FALSE;

    ucombo = data;
    gtk_signal_emit_stop_by_name(GTK_OBJECT(widget), "key_press_event");
    gnome_ucombo_hide_popup(ucombo);

    return TRUE;
}

/* This function is yanked from gtkcombo.c */
static gint
 button_press_popup(GtkWidget * widget, GdkEventButton * event, gpointer data)
{
    GnomeUcombo *ucombo;
    GtkWidget *child;

    ucombo = data;

    child = gtk_get_event_widget((GdkEvent *) event);

    /* We don't ask for button press events on the grab widget, so
     *  if an event is reported directly to the grab widget, it must
     *  be on a window outside the application (and thus we remove
     *  the popup window). Otherwise, we check if the widget is a child
     *  of the grab widget, and only remove the popup window if it
     *  is not.
     */
    if (child != widget) {
	while (child) {
	    if (child == widget)
		return FALSE;
	    child = child->parent;
	}
    }
    gnome_ucombo_hide_popup(ucombo);

    return TRUE;
}

static void position_popup(GnomeUcombo * ucombo)
{
    gint x, y;
    gint bwidth, bheight;

    gtk_widget_size_request(ucombo->popwindow, &ucombo->popwindow->requisition);

    gdk_window_get_origin(ucombo->button->window, &x, &y);
    gdk_window_get_size(ucombo->button->window, &bwidth, &bheight);

    x += bwidth - ucombo->popwindow->requisition.width;
    y += bheight;

    if (x < 0)
	x = 0;

    if (y < 0)
	y = 0;

    gtk_widget_set_uposition(ucombo->popwindow, x, y);
}

static void select_clicked(GtkWidget * widget, GnomeUcombo * ucombo)
{
    GdkCursor *cursor;

    position_popup(ucombo);
    gtk_widget_show(ucombo->popwindow);
    gtk_widget_grab_focus(ucombo->popwindow);
    gtk_grab_add(ucombo->popwindow);

    cursor = gdk_cursor_new(GDK_ARROW);

    gdk_pointer_grab(ucombo->popwindow->window, TRUE,
		     (GDK_BUTTON_PRESS_MASK
		      | GDK_BUTTON_RELEASE_MASK
		      | GDK_POINTER_MOTION_MASK),
		     NULL, cursor, GDK_CURRENT_TIME);

    gdk_cursor_destroy(cursor);
}

static void gnome_ucombo_class_init(GnomeUcomboClass * class)
{
    GtkObjectClass *object_class = (GtkObjectClass *) class;
    GtkContainerClass *container_class = (GtkContainerClass *) class;

    object_class = (GtkObjectClass *) class;

    parent_class = gtk_type_class(gtk_hbox_get_type());

    container_class->forall = gnome_ucombo_forall;

    object_class->destroy = gnome_ucombo_destroy;
}

static void gnome_ucombo_init(GnomeUcombo * ucombo)
{
}

static void gnome_ucombo_destroy(GtkObject * object)
{
    GnomeUcombo *ucombo;

    g_return_if_fail(object != NULL);
    g_return_if_fail(GNOME_IS_UCOMBO(object));

    ucombo = GNOME_UCOMBO(object);

    gtk_widget_destroy(ucombo->popwindow); /* it should destroy also popwidget as a child */

    if (GTK_OBJECT_CLASS(parent_class)->destroy)
	(*GTK_OBJECT_CLASS(parent_class)->destroy) (object);
}

static void gnome_ucombo_forall(GtkContainer * container, gboolean include_internals,
			    GtkCallback callback, gpointer callback_data)
{
    g_return_if_fail(container != NULL);
    g_return_if_fail(GNOME_IS_UCOMBO(container));
    g_return_if_fail(callback != NULL);

    /* Let GtkBox handle things only if the internal widgets need to be
     * poked.
     */
    if (include_internals)
	if (GTK_CONTAINER_CLASS(parent_class)->forall)
	    (*GTK_CONTAINER_CLASS(parent_class)->forall) (container,
						       include_internals,
							  callback,
							  callback_data);
}

static void create_children(GnomeUcombo * ucombo)
{
    GtkWidget *frame;
    GtkWidget *hbox;
    GtkWidget *arrow;

    ucombo->entry = gtk_entry_new();
    gtk_widget_set_usize(ucombo->entry, 90, 0);
    gtk_box_pack_start(GTK_BOX(ucombo), ucombo->entry, TRUE, TRUE, 0);
    gtk_widget_show(ucombo->entry);

    ucombo->button = gtk_button_new();
    gtk_object_set(GTK_OBJECT(ucombo->button), "can_focus", FALSE, NULL);
    gtk_signal_connect(GTK_OBJECT(ucombo->button), "clicked",
		       GTK_SIGNAL_FUNC(select_clicked), ucombo);
    gtk_box_pack_start(GTK_BOX(ucombo), ucombo->button, FALSE, FALSE, 0);

    hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
    gtk_container_add(GTK_CONTAINER(ucombo->button), hbox);
    gtk_widget_show(hbox);

    arrow = gtk_arrow_new(GTK_ARROW_DOWN, GTK_SHADOW_OUT);
    gtk_box_pack_start(GTK_BOX(hbox), arrow, FALSE, FALSE, 0);
    gtk_widget_show(arrow);

    gtk_widget_show(ucombo->button);

    ucombo->popwindow = gtk_window_new(GTK_WINDOW_POPUP);
    gtk_widget_set_events(ucombo->popwindow,
	  gtk_widget_get_events(ucombo->popwindow) | GDK_KEY_PRESS_MASK);
    gtk_signal_connect(GTK_OBJECT(ucombo->popwindow), "delete_event",
		       (GtkSignalFunc) delete_popup,
		       ucombo);
    gtk_signal_connect(GTK_OBJECT(ucombo->popwindow), "key_press_event",
		       (GtkSignalFunc) key_press_popup,
		       ucombo);
    gtk_signal_connect(GTK_OBJECT(ucombo->popwindow), "button_press_event",
		       (GtkSignalFunc) button_press_popup,
		       ucombo);
    gtk_window_set_policy(GTK_WINDOW(ucombo->popwindow), FALSE, TRUE, TRUE);

    frame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_IN);
    gtk_container_add(GTK_CONTAINER(ucombo->popwindow), frame);
    gtk_widget_show(frame);

    gtk_container_add(GTK_CONTAINER(frame), ucombo->popwidget);
    gtk_widget_show(ucombo->popwidget);
}

/*
 * just creates a new universal-combo widget 
 * with popup widget passed as first and only argument
 */
GtkWidget *
 gnome_ucombo_new(GtkWidget * the_popwidget)
{
    GnomeUcombo *ucombo;

    ucombo = gtk_type_new(gnome_ucombo_get_type());

    ucombo->popwidget = the_popwidget;
    create_children(ucombo);

    return GTK_WIDGET(ucombo);
}
