
#include <config.h>

#include <gnome.h>

#include <langinfo.h>
#include <stdio.h>

#include <ctype.h>

#include "data.h"
#include "actions.h"
#include "misc.h"

#include "report.h"

#include "schedule.h"

#include "interface.h"


/*****************************************
 *           Accounts Tree               *
 *****************************************/

gint interface_accounts_press_event(GtkWidget *clist, GdkEventButton *event, gpointer *data)
{
     gint row, column;
     AccountData *account;
     if (!event) {
          return FALSE;
     }
     
     if (!gtk_clist_get_selection_info(GTK_CLIST(clist), event->x, event->y, &row, &column)) {
	  return FALSE;
     }
 
     if (event->button == 3) {	 
	  /*gtk_clist_moveto (GTK_CLIST(clist), row, column, -1, -1);*/
          gtk_clist_select_row (GTK_CLIST(clist), row, column);
 
	  switch (column) {
	  case 0:   // name
	  case 2:   // flags
	       action_account_edit (NULL, NULL);
	       break;
	  case 1:   // amount
	       account = gtk_ctree_node_get_row_data(GTK_CTREE(interface.accounts_tree), 
						     GTK_CLIST(interface.accounts_tree)->selection->data);
	       action_transaction_new (NULL, account);
	       break;
	  default:  
	       break;
	  }
	  return TRUE;
     }
     
     return FALSE;
}

GtkWidget *interface_build_accounts_tree_widget(void)
{
    GtkWidget *scrolled_window;
    gchar *titles[4] = {_("Name"), _("Amount"), _("Flags"), ""};

    interface.accounts_tree = gtk_ctree_new_with_titles(4, 0, titles);
    gtk_clist_set_column_justification(GTK_CLIST(interface.accounts_tree), 1, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_column_justification(GTK_CLIST(interface.accounts_tree), 2, GTK_JUSTIFY_CENTER);
    gtk_clist_column_title_passive(GTK_CLIST(interface.accounts_tree), 0);
    gtk_clist_column_title_passive(GTK_CLIST(interface.accounts_tree), 1);
    gtk_clist_column_title_passive(GTK_CLIST(interface.accounts_tree), 2);
    gtk_clist_column_title_passive(GTK_CLIST(interface.accounts_tree), 3);
    gtk_widget_set_usize(interface.accounts_tree, 200, 50);

    gtk_signal_connect(GTK_OBJECT(interface.accounts_tree),
 		       "button_press_event",
 		       GTK_SIGNAL_FUNC(interface_accounts_press_event),
 		       NULL);   
    
    gtk_widget_show(interface.accounts_tree);

    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(scrolled_window), interface.accounts_tree);
    gtk_widget_show(scrolled_window);

    return scrolled_window;
}

void interface_build_account_node(AccountData * account)
{
     gchar flags_text[4];
     gchar amount_text[32];
     gchar *text[4];

     if (account->tree_node != NULL)
	  return;

     text[0] = account->name;
     text[1] = amount_text;
     text[2] = flags_text;
     text[3] = NULL;

     sprintf(amount_text, "%.2lf", account->amount);
     strcpy(flags_text, "");
     strcat(flags_text, account->reversed ? "r" : "");
     strcat(flags_text, account->unlimited ? "u" : "");
     if (account->parent == NULL)
	  account->tree_node =
	       gtk_ctree_insert_node(GTK_CTREE(interface.accounts_tree),
				     NULL, NULL,
				     text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
     else {
	  if (account->parent->tree_node == NULL)
	       interface_build_account_node(account->parent);

	  account->tree_node =
	       gtk_ctree_insert_node(GTK_CTREE(interface.accounts_tree),
				     account->parent->tree_node, NULL,
				     text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
     }
     gtk_ctree_node_set_row_data(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), account);
}

void interface_setup_account_node_amount(AccountData * account)
{
     gchar amount_text[32];

     g_assert(account->tree_node != NULL);

     sprintf(amount_text, "%.2lf", account->amount);
     gtk_ctree_node_set_text(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), 1, amount_text);
}

void interface_setup_account_node(AccountData *account)
{
     char flags_text[4];
     char amount_text[32];

     g_assert(account->tree_node != NULL);

     gtk_ctree_node_set_text(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), 0, account->name);

     sprintf(amount_text, "%.2lf", account->amount);
     gtk_ctree_node_set_text(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), 1, amount_text);

     strcpy(flags_text, "");
     strcat(flags_text, account->reversed ? "r" : "");
     strcat(flags_text, account->unlimited ? "u" : "");
     gtk_ctree_node_set_text(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), 2, flags_text);

     if (account->parent == NULL) {
	  gtk_ctree_move(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), NULL, NULL);
	  gtk_ctree_sort_node(GTK_CTREE(interface.accounts_tree), NULL);
     } else {
	  if (account->parent->tree_node == NULL)
	       interface_build_account_node(account->parent);

	  gtk_ctree_move(GTK_CTREE(interface.accounts_tree), GTK_CTREE_NODE(account->tree_node), 
			 account->parent->tree_node, NULL);
	  gtk_ctree_sort_node(GTK_CTREE(interface.accounts_tree), account->parent->tree_node);
     }
}

void interface_build_all_accounts(void)
{
     GList *cursor;

     gtk_clist_freeze(GTK_CLIST(interface.accounts_tree));
     gtk_clist_clear(GTK_CLIST(interface.accounts_tree));
     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
	  interface_build_account_node(AccountPtr(cursor->data));
     gtk_ctree_sort_recursive(GTK_CTREE(interface.accounts_tree), NULL);
     gtk_clist_thaw(GTK_CLIST(interface.accounts_tree));
}

void interface_setup_all_accounts_amount(void)
{
     GList *cursor;

     gtk_clist_freeze(GTK_CLIST(interface.accounts_tree));
     for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
	  interface_setup_account_node_amount(AccountPtr(cursor->data));
     gtk_clist_thaw(GTK_CLIST(interface.accounts_tree));
}

/*****************************************
 *         Transaction Registry          *
 *****************************************/

void interface_build_all_transactions(void);
GtkWidget *interface_build_transactions_registry_widget(void);

GnomeUIInfo interface_transactions_registry_press_menu[] =
{
     GNOMEUIINFO_ITEM_STOCK("_Edit...", "Edit selected transaction", 
			    action_transaction_edit, GNOME_STOCK_MENU_PROP),
     
     GNOMEUIINFO_ITEM_STOCK("_Delete", "Delete transaction", 
			    action_transaction_delete, GNOME_STOCK_MENU_TRASH),
     
     GNOMEUIINFO_END
};
 
gint interface_transactions_registry_press_event(GtkWidget *clist, GdkEventButton *event, gpointer *data)
{
     gint row, column;
     if (!event) {
          return FALSE;
     }
    
     if (!gtk_clist_get_selection_info(GTK_CLIST(clist), event->x, event->y,
 				       &row, &column)) {
	  return FALSE;
     }
 
     if (event->button == 3) {
          /*gtk_clist_moveto (GTK_CLIST(clist), row, column, -1, -1);*/
	  gtk_clist_select_row (GTK_CLIST(clist), row, column);
	  gnome_popup_menu_do_popup (gnome_popup_menu_new (interface_transactions_registry_press_menu),
 				    NULL, NULL, NULL, NULL);
	  return TRUE;
     }
    
     return FALSE;   
}
 
GList *interface_trans_sort_order = NULL;

gint interface_transactions_registry_compare(GtkCList *clist, const gconstpointer ptr1, const gconstpointer ptr2)
{
     GList *cmp_func = interface_trans_sort_order;
     gint tmp = 0;

     TransactionData *a = TransactionPtr(((GtkCListRow *)ptr1)->data);
     TransactionData *b = TransactionPtr(((GtkCListRow *)ptr2)->data);
     while ((tmp == 0) && (cmp_func != NULL)) {
	  switch (ABS(GPOINTER_TO_INT(cmp_func->data))) {
	  case 1:
	       tmp = strcoll(a->desc, b->desc);
	       break;
	  case 2:
	       tmp = g_date_compare(&(a->date), &(b->date));
	       break;
	  case 3:
	       tmp = strcoll(a->acc_from->name, b->acc_from->name);
	       break;
	  case 4:
	       tmp = strcoll(a->acc_to->name, b->acc_to->name);
	       break;
	  case 5:
	       tmp = (a->amount > b->amount) ? 1 : (a->amount == b->amount) ? 0 : -1;
	       break;
	  default:
	       g_assert_not_reached();
	  }

	  if (GPOINTER_TO_INT(cmp_func->data)<0)
	       tmp = -tmp;

	  cmp_func = g_list_next(cmp_func);
     }

     return tmp;
}

void interface_sort_transactions(void)
{
     gint row;

     gtk_clist_sort(GTK_CLIST(interface.transactions_registry));

     /* repair row field in TransactionData (QuickFIX (tm))*/
     for (row = 0; row < GTK_CLIST(interface.transactions_registry)->rows; row++) {
	  TransactionData *trans;
	  trans = TransactionPtr(gtk_clist_get_row_data(GTK_CLIST(interface.transactions_registry), row));
	  trans->row = row;				 
     }

     interface_setup_all_transactions_colors();
}

void interface_transactions_registry_select_column(GtkWidget *clist, gint column, gpointer callback_data)
{
     column++;
     if ((interface_trans_sort_order != NULL) && (ABS(GPOINTER_TO_INT(interface_trans_sort_order->data)) == column))
	  interface_trans_sort_order->data = GINT_TO_POINTER(-GPOINTER_TO_INT(interface_trans_sort_order->data));
     else {
	  interface_trans_sort_order = g_list_remove(interface_trans_sort_order, GINT_TO_POINTER(column));
	  interface_trans_sort_order = g_list_remove(interface_trans_sort_order, GINT_TO_POINTER(-column));

	  interface_trans_sort_order = g_list_prepend(interface_trans_sort_order, GINT_TO_POINTER(column));
     }

     interface_sort_transactions();
}

GtkWidget *interface_build_transactions_registry_widget(void)
{
    GtkWidget *scrolled_window;
    gchar *titles[6] = {_("Description"), _("Date"), _("From"), _("To"), _("Amount"), ""};

    interface.transactions_registry = gtk_clist_new_with_titles(6, titles);

    gtk_clist_set_column_justification(GTK_CLIST(interface.transactions_registry), 4, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_selection_mode(GTK_CLIST(interface.transactions_registry), GTK_SELECTION_MULTIPLE);
    gtk_clist_column_title_passive(GTK_CLIST(interface.transactions_registry), 5);

    gtk_signal_connect(GTK_OBJECT(interface.transactions_registry), "click_column",
		       GTK_SIGNAL_FUNC(interface_transactions_registry_select_column), NULL);

    gtk_signal_connect(GTK_OBJECT(interface.transactions_registry), "button_press_event",
 		       GTK_SIGNAL_FUNC(interface_transactions_registry_press_event), NULL);

    gtk_clist_set_compare_func(GTK_CLIST(interface.transactions_registry), interface_transactions_registry_compare);

    gtk_widget_set_usize(interface.accounts_tree, 400, 50);

    gtk_widget_show(interface.transactions_registry);

    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(scrolled_window), interface.transactions_registry);
    gtk_widget_show(scrolled_window);

    return scrolled_window;
}

void interface_build_transaction(TransactionData *trans)
{
     char amount_text[32], date_text[16];
     gchar *text[6];

     if (!trans->inbound) {
	  trans->row = -1;
	  return;
     }

     text[0] = trans->desc;
     text[1] = date_text;
     text[2] = trans->acc_from->name;
     text[3] = trans->acc_to->name;
     text[4] = amount_text;
     text[5] = NULL;

     sprintf(amount_text, "%.2lf", trans->amount);
     misc_date_strftime(date_text, 16, &(trans->date));

     trans->row = gtk_clist_append(GTK_CLIST(interface.transactions_registry), text);
     gtk_clist_set_row_data(GTK_CLIST(interface.transactions_registry), trans->row, trans);
}

void interface_setup_transaction(TransactionData *trans)
{
     if (!trans->inbound) {
	  if (trans->row != (-1)) {
	      gtk_clist_remove(GTK_CLIST(interface.transactions_registry), trans->row);
	      trans->row = -1;
	  }
     } else {
	  if (trans->row == (-1))
	       interface_build_transaction(trans);
	  else {
	       char amount_text[32], date_text[16];
	       
	       gtk_clist_set_text(GTK_CLIST(interface.transactions_registry), trans->row, 0, trans->desc);

	       misc_date_strftime(date_text, 16, &(trans->date));
	       gtk_clist_set_text(GTK_CLIST(interface.transactions_registry), trans->row, 1, date_text);

	       gtk_clist_set_text(GTK_CLIST(interface.transactions_registry), trans->row, 2, trans->acc_from->name);
	       gtk_clist_set_text(GTK_CLIST(interface.transactions_registry), trans->row, 3, trans->acc_to->name);

	       sprintf(amount_text, "%.2lf", trans->amount);
	       gtk_clist_set_text(GTK_CLIST(interface.transactions_registry), trans->row, 4, amount_text);
	  }
     }
}

void interface_setup_transaction_colors(TransactionData *trans)
{
     GdkColor color;
     gboolean highlight;

     g_assert(trans != NULL);
     g_assert(trans->row != (-1));
     g_assert((0<=interface.trans_highlight_mode) && (interface.trans_highlight_mode<=2));

     switch(interface.trans_highlight_mode) {
     case 0: 
     case 2:
	  highlight = 0;
	  break;
     case 1:
	  highlight = trans->row % 2;
	  break;
     }

     if (!highlight) 
	  gdk_color_white(gdk_colormap_get_system(), &color);
     else {
	  color.red = 34560;
	  color.green = 56736;
	  color.blue = 60160;
	  gdk_color_alloc(gdk_colormap_get_system(), &color);
     }

     gtk_clist_set_background(GTK_CLIST(interface.transactions_registry), trans->row, &color);

    /* FIXME - ???
       gdk_color_free(&color);
     */
}

void interface_setup_all_transactions_colors(void)
{
     GdkColor color1, color2;
     TransactionData *a, *b;
     gboolean highlight;
     gint i;

     g_assert((0<=interface.trans_highlight_mode) && (interface.trans_highlight_mode<=2));

     color1.red = 34560;
     color1.green = 56736;
     color1.blue = 60160;
     gdk_color_alloc(gdk_colormap_get_system(), &color1);
     gdk_color_white(gdk_colormap_get_system(), &color2);

     for (i = 0; i < GTK_CLIST(interface.transactions_registry)->rows; i++) {
	  switch(interface.trans_highlight_mode) {
	  case 0: 
	       highlight = FALSE;
	       break;
	  case 1:
	       highlight = i % 2;
	       break;
	  case 2: 
	       if (i == 0) {
		    highlight = FALSE;
		    b = TransactionPtr(gtk_clist_get_row_data(
			 GTK_CLIST(interface.transactions_registry), 0
			 ));
	       } else {
		    a = b;
		    b = TransactionPtr(gtk_clist_get_row_data(
			 GTK_CLIST(interface.transactions_registry), i
			 ));

		    switch (ABS(GPOINTER_TO_INT(interface_trans_sort_order->data))) {
		    case 1:
			 if (strcmp(a->desc, b->desc) != 0)
			      highlight = !highlight;
			 break;
		    case 2:
			 if (g_date_compare(&(a->date), &(b->date)) != 0)
			      highlight = !highlight;
			 break;
		    case 3:
			 if (strcoll(a->acc_from->name, b->acc_from->name) != 0)
			      highlight = !highlight;
			 break;
		    case 4:
			 if (strcoll(a->acc_to->name, b->acc_to->name) != 0)
			      highlight = !highlight;
			 break;
		    case 5:
			 if (a->amount != b->amount)
			      highlight = !highlight;
			 break;
		    }
	       }
	       break;
	  }

	  gtk_clist_set_background(GTK_CLIST(interface.transactions_registry), i, 
				   highlight ? &color1 : &color2);
     }
    /* FIXME - ???
       gdk_color_free(&color1);
       gdk_color_free(&color2);
     */
}

void interface_build_all_transactions(void)
{
    GList *cursor;

    gtk_clist_freeze(GTK_CLIST(interface.transactions_registry));
    gtk_clist_clear(GTK_CLIST(interface.transactions_registry));
    for (cursor = trans_list; cursor != NULL; cursor = g_list_next(cursor))
	interface_build_transaction(TransactionPtr(cursor->data));
    interface_sort_transactions();
    gtk_clist_thaw(GTK_CLIST(interface.transactions_registry));
}

void interface_setup_all_transactions(void)
{
    GList *cursor;

    gtk_clist_freeze(GTK_CLIST(interface.transactions_registry));
    for (cursor = trans_list; cursor != NULL; cursor = g_list_next(cursor))
	interface_setup_transaction(TransactionPtr(cursor->data));
    interface_sort_transactions();
    gtk_clist_thaw(GTK_CLIST(interface.transactions_registry));
}

/*****************************************
 *         Schedule List                 *
 *****************************************/

void interface_build_all_schedules(void);
GtkWidget *interface_build_schedule_list_widget(void);

GList *interface_sched_sort_order = NULL;

gint interface_schedule_list_compare(GtkCList *clist, const gconstpointer ptr1, const gconstpointer ptr2)
{
     GList *cmp_func = interface_sched_sort_order;
     gint tmp = 0;

     ScheduleData *a = SchedulePtr(((GtkCListRow *)ptr1)->data);
     ScheduleData *b = SchedulePtr(((GtkCListRow *)ptr2)->data);
     while ((tmp == 0) && (cmp_func != NULL)) {
	  switch (ABS(GPOINTER_TO_INT(cmp_func->data))) {
	  case 1:
	       tmp = strcoll(a->desc, b->desc);
	       break;
	  case 2:
	       tmp = g_date_compare(&(a->date), &(b->date));
	       break;
	  case 3:
	       tmp = strcoll(a->acc_from->name, b->acc_from->name);
	       break;
	  case 4:
	       tmp = strcoll(a->acc_to->name, b->acc_to->name);
	       break;
	  case 5:
	       tmp = (a->amount > b->amount) ? 1 : (a->amount == b->amount) ? 0 : -1;
	       break;
	  case 6:
	       tmp = (a->frequency > b->frequency) ? 1 : (a->frequency == b->frequency) ? 0 : -1;
	       break;
	  default:
	       g_assert_not_reached();
	  }

	  if (GPOINTER_TO_INT(cmp_func->data)<0)
	       tmp = -tmp;

	  cmp_func = g_list_next(cmp_func);
     }

     return tmp;
}

void interface_sort_schedules(void)
{
     gint row;

     gtk_clist_sort(GTK_CLIST(interface.schedule_list));

     /* repair row field in SchedactionData (QuickFIX (tm))*/
     for (row = 0; row < GTK_CLIST(interface.schedule_list)->rows; row++) {
	  ScheduleData *sched;
	  sched = SchedulePtr(gtk_clist_get_row_data(GTK_CLIST(interface.schedule_list), row));
	  sched->row = row;				 
     }
}

void interface_schedule_list_select_column(GtkWidget *clist, gint column, gpointer callback_data)
{
     column++;
     if ((interface_sched_sort_order != NULL) && (ABS(GPOINTER_TO_INT(interface_sched_sort_order->data)) == column))
	  interface_sched_sort_order->data = GINT_TO_POINTER(-GPOINTER_TO_INT(interface_sched_sort_order->data));
     else {
	  interface_sched_sort_order = g_list_remove(interface_sched_sort_order, GINT_TO_POINTER(column));
	  interface_sched_sort_order = g_list_remove(interface_sched_sort_order, GINT_TO_POINTER(-column));

	  interface_sched_sort_order = g_list_prepend(interface_sched_sort_order, GINT_TO_POINTER(column));
     }

     interface_sort_schedules();
}

GtkWidget *interface_build_schedule_list_widget(void)
{
    GtkWidget *scrolled_window;
    gchar *titles[8] = {_("Description"), _("Next Date"), _("From"), _("To"), _("Amount"), _("Freq"), _("Factor"),""};

    interface.schedule_list = gtk_clist_new_with_titles(8, titles);
    gtk_clist_set_column_justification(GTK_CLIST(interface.schedule_list), 4, GTK_JUSTIFY_RIGHT);
    gtk_clist_set_selection_mode(GTK_CLIST(interface.schedule_list), GTK_SELECTION_MULTIPLE);
    gtk_clist_column_title_passive(GTK_CLIST(interface.schedule_list), 5);

    gtk_signal_connect(GTK_OBJECT(interface.schedule_list), "click_column",
		       GTK_SIGNAL_FUNC(interface_schedule_list_select_column), NULL);

    gtk_clist_set_compare_func(GTK_CLIST(interface.schedule_list), interface_schedule_list_compare);

    gtk_widget_set_usize(interface.schedule_list, 400, 50);

    gtk_widget_show(interface.schedule_list);

    scrolled_window = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_container_add(GTK_CONTAINER(scrolled_window), interface.schedule_list);
    gtk_widget_show(scrolled_window);

    return scrolled_window;
}

void interface_build_schedule(ScheduleData *schedule)
{
     char amount_text[32], date_text[16], factor_text[32];
     gchar *text[8];

     text[0] = schedule->desc;
     text[1] = date_text;
     text[2] = schedule->acc_from->name;
     text[3] = schedule->acc_to->name;
     text[4] = amount_text;
     text[5] = _(frequency_option_text[schedule->frequency]);
     text[6] = factor_text;
     text[7] = NULL;

     sprintf(amount_text, "%.2lf", schedule->amount);
     misc_date_strftime(date_text, 16, &(schedule->date));
     sprintf(factor_text, "%2d", schedule->factor );
     
     schedule->row = gtk_clist_append(GTK_CLIST(interface.schedule_list), text);
     gtk_clist_set_row_data(GTK_CLIST(interface.schedule_list), schedule->row, schedule);
}

void interface_setup_schedule(ScheduleData *schedule)
{
     if (schedule->row == (-1))
	  interface_build_schedule(schedule);
     else {
	  char amount_text[32], date_text[16], factor_text[32];
    
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 0, schedule->desc);

	  misc_date_strftime(date_text, 16, &(schedule->date));
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 1, date_text);

	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 2, schedule->acc_from->name);
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 3, schedule->acc_to->name);

	  sprintf(amount_text, "%.2lf", schedule->amount);
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 4, amount_text);
	       
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 5, _(frequency_option_text[schedule->frequency]));

	  sprintf(factor_text, "%2d", schedule->factor);
	  gtk_clist_set_text(GTK_CLIST(interface.schedule_list), schedule->row, 6, factor_text);
	       
     }
}

void interface_build_all_schedules(void)
{
    GList *cursor;

    gtk_clist_freeze(GTK_CLIST(interface.schedule_list));
    gtk_clist_clear(GTK_CLIST(interface.schedule_list));
    for (cursor = schedule_list; cursor != NULL; cursor = g_list_next(cursor))
	interface_build_schedule(SchedulePtr(cursor->data));
    interface_sort_schedules();
    gtk_clist_thaw(GTK_CLIST(interface.schedule_list));
}

void interface_setup_all_schedules(void)
{
    GList *cursor;

    gtk_clist_freeze(GTK_CLIST(interface.schedule_list));
    for (cursor = schedule_list; cursor != NULL; cursor = g_list_next(cursor))
	interface_setup_schedule(SchedulePtr(cursor->data));
    interface_sort_schedules();
    gtk_clist_thaw(GTK_CLIST(interface.schedule_list));
}

/*****************************************
 *               Notebook                *
 *****************************************/

GtkWidget *interface_build_notebook(void)
{
    GtkWidget *notebook;
    
    notebook = gtk_notebook_new();

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), interface_build_accounts_tree_widget(), 
			     gtk_label_new(_("Accounts")));

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), interface_build_transactions_registry_widget(), 
			     gtk_label_new(_("Transactions")));

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), interface_build_schedule_list_widget(), 
			     gtk_label_new(_("Schedule")));

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), report_widget_new(1), 
			     gtk_label_new(_("Report")));

    gtk_widget_show(notebook);

    return notebook;
}

/*****************************************
 *              Limit Entry              *
 *****************************************/

/* callback for pressed-key event in the limit entry */
static gint interface_limit_presskey_event(GtkWidget *entry, 
					   GdkEventKey *key_event, 
					   gpointer callback_data)
{
  g_return_val_if_fail(entry != NULL, FALSE);
  g_return_val_if_fail(key_event != NULL, FALSE);

  if ((key_event->keyval != GDK_Return) && (key_event->keyval != GDK_KP_Enter))
    return FALSE;

  action_limit_apply(NULL, GINT_TO_POINTER(TRUE));
  return TRUE;
}

GtkWidget *interface_build_limit_entry(void)
{
     GtkWidget *hbox;
     GtkWidget *label;
     GtkWidget *edit_button;

     hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);

     label = gtk_label_new(_("Limit:"));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, GNOME_PAD);
     gtk_widget_show(label);

     interface.limit_entry = gtk_entry_new();
     gtk_signal_connect(GTK_OBJECT(interface.limit_entry), "key_press_event", 
     			GTK_SIGNAL_FUNC(interface_limit_presskey_event), NULL);
     gtk_box_pack_start(GTK_BOX(hbox), interface.limit_entry, TRUE, TRUE, 0);
     gtk_widget_show(interface.limit_entry);

     edit_button = gtk_button_new_with_label(_("Edit..."));
     gtk_signal_connect(GTK_OBJECT(edit_button), "clicked", GTK_SIGNAL_FUNC(action_limit_edit), NULL);
     gtk_box_pack_start(GTK_BOX(hbox), edit_button, FALSE, FALSE, 0);
     gtk_widget_show(edit_button);

     gtk_widget_show(hbox);

     return hbox;
}

void interface_setup_limit(gchar *expr)
{
     gtk_entry_set_text(GTK_ENTRY(interface.limit_entry), expr);
}

gchar *interface_get_limit(void)
{
     return gtk_entry_get_text(GTK_ENTRY(interface.limit_entry));
}

/*****************************************
 *           User Box                    *
 *****************************************/

GtkWidget *interface_build_user_box(void)
{
    GtkWidget *user_box;

    user_box = gtk_vbox_new(FALSE, GNOME_PAD);
    gtk_container_set_border_width(GTK_CONTAINER(user_box), GNOME_PAD);

    gtk_box_pack_start(GTK_BOX(user_box), interface_build_limit_entry(), FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(user_box), interface_build_notebook(), TRUE, TRUE, 0);

    gtk_widget_show(user_box);

    return user_box;
}

/*****************************************
 *            Main Window                *
 *****************************************/

#include "menu.h"
#include "toolbar.h"

gint interface_quit_event(GtkWidget* widget, GtkWidget* event, gpointer data)
{
    int ret = action_do_you_want_to_save();
    switch (ret) {
    case 0:			/* yes */
	action_save(widget, action_quit);
	return TRUE;
    case 1:			/* no */
	gtk_main_quit();
	return FALSE;
    case 2:			/* cancel */
	return TRUE;
    default:			
	 g_assert_not_reached();
	 return FALSE;
    }
}

void interface_build_application(void)
{
    interface.application = gnome_app_new("Emma", "Emma");

    gtk_window_set_default_size(GTK_WINDOW(interface.application), 600, 450);

    gtk_signal_connect(GTK_OBJECT(interface.application), "delete_event", 
		       GTK_SIGNAL_FUNC(interface_quit_event), NULL);

    menu_build(GNOME_APP(interface.application));
    toolbar_build(GNOME_APP(interface.application));

    gnome_app_set_contents(GNOME_APP(interface.application), interface_build_user_box());

    gtk_widget_show(interface.application);
}

/*****************************************
 *           INTERFACE                   *
 *****************************************/

void interface_build(void)
{
    interface_build_application();
    interface_setup_filename("");
}

void interface_free(void)
{
     g_list_free(interface_trans_sort_order);
     g_list_free(interface_sched_sort_order);
}

/*****************************************
 *             OTHERS                    *
 *****************************************/

void interface_setup_filename(const gchar *new_filename)
{
     if (g_path_is_absolute(new_filename) || (strcmp(new_filename, "") == 0))
	  strcpy(interface.filename, new_filename);
     else {
	  gchar *curdir = g_get_current_dir();
	  strcpy(interface.filename, curdir);
	  strcat(interface.filename, "/"); /* <- FIXME - is this really necesaary? */
	  strcat(interface.filename, new_filename);
     }

    /* setup main widow's title */
    if (strcmp(interface.filename, "") == 0)
	gtk_window_set_title(GTK_WINDOW(interface.application), "Emma");
    else {
	char title[1024] = "Emma - ";
	gtk_window_set_title(GTK_WINDOW(interface.application), strcat(title, g_basename(interface.filename)));
    }
}




