#include <config.h>

#include <gnome.h>

#include <langinfo.h>

#include "acc_entry.h"
#include "date_entry.h"
#include "py_module.h"

#include "misc.h"

#include "limit_dlg.h"


struct _LimitDlg {
     GtkWidget *dialog;

     GtkWidget *entry;

     GtkWidget *acc_entry;
     GtkWidget *date_entry;
     GtkWidget *func_name_combo;
     GtkWidget *func_desc_label;
} limit_dlg;;

GtkWidget *limit_dlg_build_entry(const gchar *default_limit)
{
     GtkWidget *hbox;
     GtkWidget *label;

     hbox = gtk_hbox_new(FALSE, 0);

     label = gtk_label_new(_("Expression:"));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);

     limit_dlg.entry = gtk_entry_new();
     gtk_entry_set_text(GTK_ENTRY(limit_dlg.entry), default_limit);
     gtk_entry_set_max_length(GTK_ENTRY(limit_dlg.entry), PY_EXPRESSION_MAX_SIZE-1);
     gtk_box_pack_start(GTK_BOX(hbox), limit_dlg.entry, TRUE, TRUE, GNOME_PAD);
     gtk_widget_show(limit_dlg.entry);

     gtk_widget_set_usize(hbox, 300, 10);
     gtk_widget_show(hbox);

     return hbox;
}

void limit_dlg_account_insert(GtkWidget *widget, gpointer callback_data)
{
     AccountData *account;
     gchar text[ACCOUNT_NAME_SIZE+2];
     gint pos;

     if (acc_entry_get_account(limit_dlg.acc_entry, &account) || (account == NULL))
	  return;

     strcpy(text, "\"");
     strcat(text, account->name);
     strcat(text, "\"");

     pos = gtk_editable_get_position(GTK_EDITABLE(limit_dlg.entry));
     gtk_editable_insert_text(GTK_EDITABLE(limit_dlg.entry), text, strlen(text), &pos);
     gtk_entry_set_position(GTK_ENTRY(limit_dlg.entry), pos);
}

GtkWidget *limit_dlg_build_account(void)
{
     GtkWidget *hbox, *vbox;
     GtkWidget *insert_button;
     GtkWidget *separator;

     vbox = gtk_vbox_new(FALSE, GNOME_PAD);

     limit_dlg.acc_entry = acc_entry_new(FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), limit_dlg.acc_entry, FALSE, FALSE, 0);
     gtk_widget_show(limit_dlg.acc_entry);

     separator = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
     gtk_widget_show(separator);

     hbox = gtk_hbox_new(TRUE, GNOME_PAD);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD);

     insert_button = gtk_button_new_with_label(_("Insert"));
     gtk_box_pack_start(GTK_BOX(hbox), insert_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(insert_button), "clicked", GTK_SIGNAL_FUNC(limit_dlg_account_insert), NULL);
     gtk_widget_show(insert_button);

     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_widget_show(hbox);

     gtk_widget_show(vbox);

     return vbox;
}

void limit_dlg_date_exact(GtkWidget *widget, gpointer what_to_do)
{
     GDate date;
     g_date_set_current(&date);

     switch (GPOINTER_TO_INT(what_to_do)) {
     case 0 : /* year */
	  g_date_set_month(&date, G_DATE_JANUARY);
     case 1 : /* month */
	  g_date_set_day(&date, 1);
     case 2 : /* day */
	  break;
     case 3 : /* week */
	  g_date_subtract_days(&date, g_date_weekday(&date) - 1);
	  break;
     default:
	  g_assert_not_reached();
     }

     date_entry_set_date(limit_dlg.date_entry, &date);
}

void limit_dlg_date_insert(GtkWidget *widget, gpointer callback_data)
{
     gchar text[18], date_text[16];
     gint pos;
     GDate date;

     if (date_entry_get_date(limit_dlg.date_entry, &date))
	  return;

     misc_date_strftime(date_text, 16, &date);
     strcpy(text, "\"");
     strcat(text, date_text);
     strcat(text, "\"");
	  
     pos = gtk_editable_get_position(GTK_EDITABLE(limit_dlg.entry));
     gtk_editable_insert_text(GTK_EDITABLE(limit_dlg.entry), text, strlen(text), &pos);
     gtk_entry_set_position(GTK_ENTRY(limit_dlg.entry), pos);
}

GtkWidget *limit_dlg_build_date(void)
{
     GtkWidget *hbox, *vbox;
     GtkWidget *label;
     GtkWidget *insert_button;
     GtkWidget *day_button, *month_button, *year_button, *week_button;
     GtkWidget *separator;
     GDate date;

     vbox = gtk_vbox_new(FALSE, GNOME_PAD);

     limit_dlg.date_entry = date_entry_new();
     g_date_set_current(&date);
     date_entry_set_date(limit_dlg.date_entry, &date);
     gtk_box_pack_start(GTK_BOX(vbox), limit_dlg.date_entry, FALSE, FALSE, 0);
     gtk_widget_show(limit_dlg.date_entry);

     hbox = gtk_hbox_new(TRUE, GNOME_PAD);

     label = gtk_label_new(_("Begining of"));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
     gtk_widget_show(label);

     day_button = gtk_button_new_with_label(_("Today"));
     gtk_box_pack_start(GTK_BOX(hbox), day_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(day_button), "clicked", 
			GTK_SIGNAL_FUNC(limit_dlg_date_exact), GINT_TO_POINTER(2));
     gtk_widget_show(day_button);

     month_button = gtk_button_new_with_label(_("Month"));
     gtk_box_pack_start(GTK_BOX(hbox), month_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(month_button), "clicked", 
			GTK_SIGNAL_FUNC(limit_dlg_date_exact), GINT_TO_POINTER(1));
     gtk_widget_show(month_button);

     year_button = gtk_button_new_with_label(_("Year"));
     gtk_box_pack_start(GTK_BOX(hbox), year_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(year_button), "clicked", 
			GTK_SIGNAL_FUNC(limit_dlg_date_exact), GINT_TO_POINTER(0));
     gtk_widget_show(year_button);

     week_button = gtk_button_new_with_label(_("Week"));
     gtk_box_pack_start(GTK_BOX(hbox), week_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(week_button), "clicked", 
			GTK_SIGNAL_FUNC(limit_dlg_date_exact), GINT_TO_POINTER(3));
     gtk_widget_show(week_button);

     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_widget_show(hbox);

     separator = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
     gtk_widget_show(separator);

     hbox = gtk_hbox_new(TRUE, GNOME_PAD);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD);

     insert_button = gtk_button_new_with_label(_("Insert"));
     gtk_box_pack_start(GTK_BOX(hbox), insert_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(insert_button), "clicked", GTK_SIGNAL_FUNC(limit_dlg_date_insert), NULL);
     gtk_widget_show(insert_button);

     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_widget_show(hbox);

     gtk_widget_show(vbox);

     return vbox;
}

void limit_dlg_function_insert(GtkWidget *widget, gpointer callback_data)
{
     gint pos;
     gchar text[PY_EXPRESSION_MAX_SIZE];

     strcpy(text, gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(limit_dlg.func_name_combo)->entry)));
     strcat(text, "()");

     pos = gtk_editable_get_position(GTK_EDITABLE(limit_dlg.entry));
     gtk_editable_insert_text(GTK_EDITABLE(limit_dlg.entry), text, strlen(text), &pos);
     gtk_entry_set_position(GTK_ENTRY(limit_dlg.entry), pos-1);
}

void limit_dlg_function_changed(GtkWidget *widget, gpointer callback_data)
{
     gchar *text;
     GList *cursor;
     
     text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(limit_dlg.func_name_combo)->entry));
     for (cursor = py_module_func_desc; cursor != NULL; cursor = g_list_next(cursor)) {
	  PyModuleFuncDesc *func_desc = PyModuleFuncDescPtr(cursor->data);
	  if (strcmp(text, func_desc->name) == 0) {
	       gtk_label_set_text(GTK_LABEL(limit_dlg.func_desc_label), func_desc->desc);
	       break;
	  }
     }
}

GtkWidget *limit_dlg_build_functions(void)
{
     GtkWidget *hbox, *vbox;
     GtkWidget *insert_button;
     GtkWidget *separator;
     GtkWidget *frame;
     
     GList *combo_strings = NULL, *cursor;

     vbox = gtk_vbox_new(FALSE, GNOME_PAD);

     limit_dlg.func_desc_label = gtk_label_new("");
     gtk_label_set_line_wrap(GTK_LABEL(limit_dlg.func_desc_label), TRUE);

     limit_dlg.func_name_combo = gtk_combo_new();
     gtk_editable_set_editable(GTK_EDITABLE(GTK_COMBO(limit_dlg.func_name_combo)->entry), FALSE);
     gtk_signal_connect(GTK_OBJECT(GTK_COMBO(limit_dlg.func_name_combo)->entry), "changed", 
			GTK_SIGNAL_FUNC(limit_dlg_function_changed), NULL);
     for (cursor = py_module_func_desc; cursor != NULL; cursor = g_list_next(cursor))
	  combo_strings = g_list_prepend(combo_strings, PyModuleFuncDescPtr(cursor->data)->name);
     combo_strings = g_list_sort(combo_strings, (GCompareFunc)strcoll);
     gtk_combo_set_popdown_strings(GTK_COMBO(limit_dlg.func_name_combo), combo_strings);
     g_list_free(combo_strings);
     gtk_box_pack_start(GTK_BOX(vbox), limit_dlg.func_name_combo, FALSE, FALSE, 0);
     gtk_widget_show(limit_dlg.func_name_combo);

     frame = gtk_frame_new(_("Description"));
     gtk_container_add(GTK_CONTAINER(frame), limit_dlg.func_desc_label);
     gtk_widget_show(limit_dlg.func_desc_label);

     gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
     gtk_widget_show(frame);
     
     separator = gtk_hseparator_new();
     gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 0);
     gtk_widget_show(separator);

     hbox = gtk_hbox_new(TRUE, GNOME_PAD);
     gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD);

     insert_button = gtk_button_new_with_label(_("Insert"));
     gtk_box_pack_start(GTK_BOX(hbox), insert_button, FALSE, FALSE, 0);
     gtk_signal_connect(GTK_OBJECT(insert_button), "clicked", GTK_SIGNAL_FUNC(limit_dlg_function_insert), NULL);
     gtk_widget_show(insert_button);

     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
     gtk_widget_show(hbox);

     gtk_widget_show(vbox);

     return vbox;
}

GtkWidget *limit_dlg_build_notebook(void)
{
    GtkWidget *notebook;

    notebook = gtk_notebook_new();

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), limit_dlg_build_account(), gtk_label_new(_("Account")));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), limit_dlg_build_date(), gtk_label_new(_("Date")));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), limit_dlg_build_functions(), gtk_label_new(_("Function")));
     
    gtk_widget_show(notebook);

    return notebook;

}

void limit_dlg_build(const gchar *title, const gchar *default_limit)
{
    limit_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(limit_dlg.dialog)->vbox), 
		       limit_dlg_build_entry(default_limit), 
		       FALSE, FALSE, GNOME_PAD);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(limit_dlg.dialog)->vbox), 
		       limit_dlg_build_notebook(), 
		       TRUE, TRUE, 0);
}

gboolean limit_dlg_execute(const gchar *title, gchar *limit_text)
{
     gint ret;

     limit_dlg_build(title, limit_text);
     ret = gnome_dialog_run(GNOME_DIALOG(limit_dlg.dialog));

     switch (ret) {
     case 0:		/* OK */
	  strcpy(limit_text, gtk_entry_get_text(GTK_ENTRY(limit_dlg.entry)));
     case 1:		/* Cancel */
	  gnome_dialog_close(GNOME_DIALOG(limit_dlg.dialog));
     case -1:		/* Esc or "closed window" */
	  break;
     default:
	  g_assert_not_reached();
     }

     return ret == 0; /* Ok */
}






