
#include <config.h>

#include <gnome.h>

#include <locale.h>
#include <errno.h>

#include "data.h"
#include "py_module.h"
#include "report.h"

#include "actions.h"
#include "interface.h"
#include "menu.h"
#include "prefs.h"

#include "sm.h"


static void popt_callback(poptContext ctx, 
			  enum poptCallbackReason reason, 
			  const struct poptOption *opt, 
			  char *arg, 
			  void *data)
{
     switch (opt->shortName) {
     case 'i' :
	  py_module_import_module(arg);
	  break;
     case 'p' :
	  py_module_include_file(arg, TRUE);
	  break;
     default:
	  g_assert_not_reached();
     }
}

static const struct poptOption popt_options[] = {
    {NULL, '\0', POPT_ARG_CALLBACK, &popt_callback, 0 },

    {"import", 'i', POPT_ARG_STRING, NULL, 0, 
     N_("Import python module"), N_("MODULE")},

    {"parse", 'p', POPT_ARG_STRING, NULL, 0, 
     N_("Parse a specyfied python source file"), N_("FILE")},

    {NULL, '\0', POPT_ARG_STRING, NULL, 0, 
     N_("Open a specyfied file"), N_("FILE")},

    {NULL, '\0', 0, NULL, 0}
};

int main(int argc, char *argv[])
{
     poptContext popt_context;
     char **args;

     /* i18n settings */
     setlocale(LC_ALL, "");

     bindtextdomain(PACKAGE, GNOMELOCALEDIR);
     textdomain(PACKAGE);

     /* init gnome library and deal with command line args */
     gnome_init_with_popt_table(PACKAGE, VERSION, 
				argc, argv, 
				popt_options, 0, &popt_context);
     /* launch the modules */
     menu_init();
     interface_build();

     prefs_load();

     data_init();
     report_init();
     py_module_init();

     sm_init();

     /* parse arguments */
     args = poptGetArgs(popt_context);
     if (strcmp(interface.filename, "") == 0) {
	  if (args && args[0])
	       action_open_direct(args[0]);
	  else
	       prefs_startup_open();
     }
     poptFreeContext(popt_context);

     /* main loop */
     gtk_main();

     /* destroy the modules */
     py_module_done();
     report_done();
     data_done();

     prefs_save();

     interface_free();
     menu_done();

     /* the end */
     return 0;
}


