#include <config.h>

#include <ctype.h>
#include <time.h>
#include <langinfo.h>

#include "data.h"

#include "misc.h"


void g_date_set_current(GDate *date)
{
     time_t time_tmp;
     struct tm *tm_tmp;
     
     time_tmp = time(NULL);
     tm_tmp = localtime(&time_tmp);

     g_date_set_dmy(date, 
		    tm_tmp->tm_mday, tm_tmp->tm_mon+1, tm_tmp->tm_year+1900);
}

gboolean only_white_space(const gchar *text)
{
     while (*text != '\0') {
	  if (!isspace(*text))
	       return FALSE;

	  text++;
     }

     return TRUE;
}

void translate_uiinfo(GnomeUIInfo *uiinfo)
{
  while (uiinfo->type != GNOME_APP_UI_ENDOFINFO) {
    //g_free(uiinfo->label);
    uiinfo->label = _(uiinfo->label);
	
    // g_free(uiinfo->hint);
    uiinfo->hint = _(uiinfo->hint);

    uiinfo++;
  }
}

static GtkCTreeNode *misc_build_account_node(GtkWidget *ctree, 
					     AccountData * account, 
					     GtkCTreeNode *root)
{
  if (account->extra_data == NULL) {
    gchar *text[1];
    text[0] = account->name;

    if (account->parent == 0)
      account->extra_data =
	gtk_ctree_insert_node(GTK_CTREE(ctree),
			      root, NULL,
			      text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
    else
      account->extra_data =
	gtk_ctree_insert_node(GTK_CTREE(ctree),
			      misc_build_account_node(ctree, account->parent, root), NULL,
			      text, 0, NULL, NULL, NULL, NULL, FALSE, FALSE);
  }
  return GTK_CTREE_NODE(account->extra_data);
}

GtkWidget *misc_build_ctree(GList *acc_list, gchar *root_text)
{
    GtkWidget *ctree;
    GtkCTreeNode *root;
    GList *cursor;

    gchar *titles[1] = {"Name"};
    ctree = gtk_ctree_new_with_titles(1, 0, titles);
    gtk_clist_column_titles_hide(GTK_CLIST(ctree));

    /* add the additional 'none' node */
    if (root_text != NULL) {
	    gchar *text[1] = { root_text };
	    root = gtk_ctree_insert_node(GTK_CTREE(ctree),
					 NULL, NULL,
					 text, 0, NULL, NULL, NULL, NULL, FALSE, TRUE);
    } else
      root = NULL;

    /* we will use AccountData.extra_data as pointer to tree nodes 
       we have to set them to NULL first */
    for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor))
      AccountPtr(cursor->data)->extra_data = NULL;

    /* build all tree nodes */
    for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) {
      misc_build_account_node(ctree, AccountPtr(cursor->data), root);
      gtk_ctree_node_set_row_data(GTK_CTREE(ctree),
				  GTK_CTREE_NODE(AccountPtr(cursor->data)->extra_data),
				  cursor->data);
    }
    gtk_ctree_sort_recursive(GTK_CTREE(ctree), NULL);

    /* set optimal (that is maximum that can happen) 
       width of the ctree column widget */
    gtk_ctree_expand_recursive(GTK_CTREE(ctree), NULL);
    gtk_clist_set_column_width(GTK_CLIST(ctree), 0, 
			  gtk_clist_optimal_column_width(GTK_CLIST(ctree), 0));
    gtk_ctree_collapse_recursive(GTK_CTREE(ctree), NULL);

    /* FIXME - actually I could do auto_resize, but then 
     * the dropdownbox in acc_entry moves to the left (why?) 
     * gtk_clist_set_column_auto_resize(GTK_CLIST(ctree), 0, TRUE); */

    gtk_widget_show(ctree);

    return ctree;
}

gchar misc_date_fmt[MISC_DATE_FMT_SIZE] = "%x";
void misc_date_strftime(gchar *string, gsize n, GDate *date)
{
  g_date_strftime(string, n, misc_date_fmt, date);
}

