#include <config.h>

#include <gnome.h>

#include <ctype.h>
#include <langinfo.h>

#include "interface.h"
#include "actions.h"
#include "py_module.h"

#include "menu.h"

#include "misc.h"

#include "prefs.h"

void prefs_load_accounts_tree(void)
{
    gnome_config_push_prefix("/emma/Accounts Tree/");

    gtk_clist_set_column_width(GTK_CLIST(interface.accounts_tree), 0, gnome_config_get_int("Tree Width=150"));
    gtk_clist_set_column_width(GTK_CLIST(interface.accounts_tree), 1, gnome_config_get_int("Amount Width=70"));
    gtk_clist_set_column_width(GTK_CLIST(interface.accounts_tree), 2, gnome_config_get_int("Flags Width=35"));

    gnome_config_pop_prefix();
}

void prefs_load_transactions_registry(void)
{
    gchar *sort_order;
    gint i, highlight_mode;

    gnome_config_push_prefix("/emma/Transactions Registry/");

    gtk_clist_set_column_width(GTK_CLIST(interface.transactions_registry), 0, gnome_config_get_int("Desc Width=160"));
    gtk_clist_set_column_width(GTK_CLIST(interface.transactions_registry), 1, gnome_config_get_int("Date Width=80"));
    gtk_clist_set_column_width(GTK_CLIST(interface.transactions_registry), 2, 
			       gnome_config_get_int("AccFrom Width=90"));
    gtk_clist_set_column_width(GTK_CLIST(interface.transactions_registry), 3, 
			       gnome_config_get_int("AccTo Width=90"));
    gtk_clist_set_column_width(GTK_CLIST(interface.transactions_registry), 4, 
			       gnome_config_get_int("Amount Width=70"));

    sort_order = gnome_config_get_string("Sort Order=B");
    for (i = 0; sort_order[i] != '\0'; i++) {
	 gint sortby;

	 if (!isalpha(sort_order[i]))
	      continue;

	 if (isupper(sort_order[i])) {
	      sortby = -(sort_order[i] - 'A' + 1);
	 } else {
	      sortby = (sort_order[i] - 'a' + 1);
	 }

	 interface_trans_sort_order = g_list_prepend(interface_trans_sort_order, GINT_TO_POINTER(sortby));
    }
    interface_trans_sort_order = g_list_reverse(interface_trans_sort_order);
    g_free(sort_order);

    highlight_mode = gnome_config_get_int("Highlight mode=0");
    gtk_check_menu_item_set_active(
	 GTK_CHECK_MENU_ITEM(menu_get_highlight_list()[highlight_mode].widget), TRUE);

    gnome_config_pop_prefix();
}

void prefs_load_schedule_list(void)
{
     gchar *sort_order;
     gint i;

     gnome_config_push_prefix("/emma/Schedule List/");

     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 0, gnome_config_get_int("Desc Width=150"));
     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 1, gnome_config_get_int("Date Width=80"));
     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 2, gnome_config_get_int("AccFrom Width=90"));
     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 3, gnome_config_get_int("AccTo Width=90"));
     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 4, gnome_config_get_int("Amount Width=70"));
     gtk_clist_set_column_width(GTK_CLIST(interface.schedule_list), 5, gnome_config_get_int("Freq Width=50"));

     sort_order = gnome_config_get_string("Sort Order=B");
     for (i = 0; sort_order[i] != '\0'; i++) {
	  gint sortby;
	  
	  if (!isalpha(sort_order[i]))
	       continue;

	  if (isupper(sort_order[i])) {
	       sortby = -(sort_order[i] - 'A' + 1);
	  } else {
	       sortby = (sort_order[i] - 'a' + 1);
	  }

	  interface_sched_sort_order = g_list_prepend(interface_sched_sort_order, GINT_TO_POINTER(sortby));
     }
     interface_sched_sort_order = g_list_reverse(interface_sched_sort_order);
     g_free(sort_order);

     gnome_config_pop_prefix();
}

void prefs_load_limit(void)
{
     gchar *text;

     text = gnome_config_get_string("/emma/Startup/Limit=");
     interface_setup_limit(text);
     g_free(text);
}

void prefs_load_date_fmt(void)
{
  gchar *path;
  gchar *format;

  path = g_strdup_printf("/emma/Misc/DateFormat=%s", nl_langinfo(D_FMT));
  format = gnome_config_get_string(path);

  strncpy(misc_date_fmt, format, MISC_DATE_FMT_SIZE-1);

  g_free(format);
  g_free(path);
}

void prefs_load(void)
{
     prefs_load_accounts_tree();
     prefs_load_transactions_registry();
     prefs_load_schedule_list();
     prefs_load_limit();
     prefs_load_date_fmt();
}

void prefs_save_accounts_tree(void)
{
    gnome_config_push_prefix("/emma/Accounts Tree/");

    gnome_config_set_int("Tree Width", GTK_CLIST(interface.transactions_registry)->column[0].width);
    gnome_config_set_int("Amount Width", GTK_CLIST(interface.transactions_registry)->column[1].width);
    gnome_config_set_int("Flags Width", GTK_CLIST(interface.transactions_registry)->column[2].width);

    gnome_config_pop_prefix();
}

void prefs_save_transactions_registry(void)
{
     GList *cursor;
     gchar sort_order[20]; /* FIXME - those fixed size strings aren't a GoodThing (tm) , right? */
     gint i;

     gnome_config_push_prefix("/emma/Transactions Registry/");

     gnome_config_set_int("Desc Width", GTK_CLIST(interface.transactions_registry)->column[0].width);
     gnome_config_set_int("Date Width", GTK_CLIST(interface.transactions_registry)->column[1].width);
     gnome_config_set_int("AccFrom Width", GTK_CLIST(interface.transactions_registry)->column[2].width);
     gnome_config_set_int("AccTo Width", GTK_CLIST(interface.transactions_registry)->column[3].width);
     gnome_config_set_int("Amount Width", GTK_CLIST(interface.transactions_registry)->column[4].width);

     i = 0;
     for (i = 0, cursor = interface_trans_sort_order; cursor != NULL; cursor = g_list_next(cursor)) {
	  gint sortby = GPOINTER_TO_INT(cursor->data);
	  sort_order[i] = sortby < 0 ? 'A' - 1 - sortby : 'a' - 1 + sortby;
	  i++;
     }
     sort_order[i] = '\0';
     gnome_config_set_string("Sort Order", sort_order);

     gnome_config_set_int("Highlight mode", interface.trans_highlight_mode);

     gnome_config_pop_prefix();
}

void prefs_save_schedule_list(void)
{
     GList *cursor;
     gchar sort_order[20]; /* FIXME - those fixed size strings aren't a GoodThing (tm) , right? */
     gint i;

     gnome_config_push_prefix("/emma/Schedule List/");

     gnome_config_set_int("Desc Width", GTK_CLIST(interface.schedule_list)->column[0].width);
     gnome_config_set_int("Date Width", GTK_CLIST(interface.schedule_list)->column[1].width);
     gnome_config_set_int("AccFrom Width", GTK_CLIST(interface.schedule_list)->column[2].width);
     gnome_config_set_int("AccTo Width", GTK_CLIST(interface.schedule_list)->column[3].width);
     gnome_config_set_int("Amount Width", GTK_CLIST(interface.schedule_list)->column[4].width);
     gnome_config_set_int("Freq Width", GTK_CLIST(interface.schedule_list)->column[5].width);

     i = 0;
     for (i = 0, cursor = interface_sched_sort_order; cursor != NULL; cursor = g_list_next(cursor)) {
	  gint sortby = GPOINTER_TO_INT(cursor->data);
	  sort_order[i] = sortby < 0 ? 'A' - 1 - sortby : 'a' - 1 + sortby;
	  i++;
     }
     sort_order[i] = '\0';
     gnome_config_set_string("Sort Order", sort_order);

     gnome_config_pop_prefix();
}

void prefs_save_date_fmt(void)
{
  gnome_config_set_string("/emma/Misc/DateFormat", misc_date_fmt);
}

void prefs_save(void)
{
     prefs_save_accounts_tree();
     prefs_save_transactions_registry();
     prefs_save_schedule_list();
     prefs_save_date_fmt();
     gnome_config_sync();
}

void prefs_startup_open(void)
{
     gboolean none;
     gchar *text;

     text = gnome_config_get_string_with_default("/emma/Startup/Open=", &none);
     if (!none)
	  action_open_direct(text);
     g_free(text);
}

