#include <config.h>

#include <gnome.h>

#include <langinfo.h>

#include "limit_dlg.h"
#include "py_module.h"
#include "misc.h"

#include "prefs_dlg.h"

struct _PrefsDlg {
  GtkWidget *dialog;

  GtkWidget *file_entry;
  GtkWidget *limit_entry;
  GtkWidget *date_fmt_entry;

  GtkWidget *depth_entry;
} prefs_dlg;;

void prefs_dlg_edit_limit(GtkWidget *widget, gpointer callback_data)
{
     gchar expr[PY_EXPRESSION_MAX_SIZE];
     strcpy(expr, gtk_entry_get_text(GTK_ENTRY(prefs_dlg.limit_entry)));

     if (limit_dlg_execute(_("Edit default limit..."), expr))
	  gtk_entry_set_text(GTK_ENTRY(prefs_dlg.limit_entry), expr);
}

GtkWidget *prefs_dlg_build_on_startup(void)
{
     GtkWidget *frame;
     GtkWidget *table;
     GtkWidget *label;
     GtkWidget *edit_button;
     gchar *text;

     frame = gtk_frame_new(_("General"));
     
     table = gtk_table_new(3, 3, FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD);

     prefs_dlg.file_entry = gnome_file_entry_new("EmmaPreferencesFileOnOpenHistoryId",_("Browse..."));
     text = gnome_config_get_string("/emma/Startup/Open=");
     gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prefs_dlg.file_entry))), text);
     g_free(text);
     gtk_widget_show(prefs_dlg.file_entry);
     gtk_table_attach(GTK_TABLE(table), prefs_dlg.file_entry, 
		      1, 3, 0, 1, GTK_FILL, 0, 3, 3);

     prefs_dlg.limit_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(prefs_dlg.limit_entry), 
			      PY_EXPRESSION_MAX_SIZE-1);
     text = gnome_config_get_string("/emma/Startup/Limit=");
     gtk_entry_set_text(GTK_ENTRY(prefs_dlg.limit_entry), text);
     g_free(text);
     gtk_widget_show(prefs_dlg.limit_entry);
     gtk_table_attach(GTK_TABLE(table), prefs_dlg.limit_entry, 
		      1, 2, 1, 2, GTK_FILL | GTK_EXPAND, 0, 3, 3);

     prefs_dlg.date_fmt_entry = gtk_entry_new();
     gtk_entry_set_max_length(GTK_ENTRY(prefs_dlg.date_fmt_entry), 
			      MISC_DATE_FMT_SIZE-1);
     gtk_entry_set_text(GTK_ENTRY(prefs_dlg.date_fmt_entry), misc_date_fmt);
     gtk_widget_show(prefs_dlg.date_fmt_entry);
     gtk_table_attach(GTK_TABLE(table), prefs_dlg.date_fmt_entry, 
		      1, 3, 2, 3, GTK_FILL | GTK_EXPAND, 0, 3, 3);

     label = gtk_label_new(_("File to open on startup:"));
     gtk_widget_show(label);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

     label = gtk_label_new(_("Default limit:"));
     gtk_widget_show(label);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);

     label = gtk_label_new(_("Date format:"));
     gtk_widget_show(label);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 3, 3);

     edit_button = gtk_button_new_with_label(_("Edit..."));
     gtk_signal_connect(GTK_OBJECT(edit_button), "clicked", GTK_SIGNAL_FUNC(prefs_dlg_edit_limit), NULL);
     gtk_table_attach(GTK_TABLE(table), edit_button, 2, 3, 1, 2, GTK_FILL, 0, 3, 3);
     gtk_widget_show(edit_button);

     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_widget_show(table);

     gtk_widget_show(frame);

     return frame;
}

GtkWidget *prefs_dlg_build_on_open(void)
{
     GtkWidget *frame;
     GtkWidget *table;
     GtkWidget *label;
     gchar *text;

     frame = gtk_frame_new(_("On open"));
     
     table = gtk_table_new(2, 1, FALSE);
     gtk_container_set_border_width(GTK_CONTAINER(table), GNOME_PAD);

     prefs_dlg.depth_entry = gtk_entry_new();
     text = gnome_config_get_string("/emma/Open/Expand Accounts To Depth=1");
     gtk_entry_set_text(GTK_ENTRY(prefs_dlg.depth_entry), text);
     g_free(text);
     gtk_widget_show(prefs_dlg.depth_entry);
     gtk_table_attach(GTK_TABLE(table), prefs_dlg.depth_entry, 1, 2, 0, 1, GTK_FILL | GTK_EXPAND, 0, 3, 3);

     label = gtk_label_new(_("Expand accounts to depth:"));
     gtk_widget_show(label);
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

     gtk_container_add(GTK_CONTAINER(frame), table);
     gtk_widget_show(table);

     gtk_widget_show(frame);

     return frame;
}

void prefs_dlg_build(void)
{
    prefs_dlg.dialog = gnome_dialog_new(_("Preferences..."), GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(prefs_dlg.dialog)->vbox), 
		       prefs_dlg_build_on_startup(), 
		       FALSE, FALSE, GNOME_PAD);

    gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(prefs_dlg.dialog)->vbox), 
		       prefs_dlg_build_on_open(), 
		       FALSE, FALSE, GNOME_PAD);
}

void prefs_dlg_main(void)
{
     gchar *filename;
     gint depth = -1;

     filename = gnome_file_entry_get_full_path(GNOME_FILE_ENTRY(prefs_dlg.file_entry), TRUE);
     if (filename != NULL)
	  gnome_config_set_string("/emma/Startup/Open", filename);

     gnome_config_set_string("/emma/Startup/Limit", 
	     gtk_entry_get_text(GTK_ENTRY(prefs_dlg.limit_entry)));
     
     strncpy(misc_date_fmt, 
	     gtk_entry_get_text(GTK_ENTRY(prefs_dlg.date_fmt_entry)),
	     MISC_DATE_FMT_SIZE);

     depth = atoi(gtk_entry_get_text(GTK_ENTRY(prefs_dlg.depth_entry)));
     if (depth != (-1))
	  gnome_config_set_int("/emma/Open/Expand Accounts To Depth=1", depth);

     gnome_config_sync();
}

gboolean prefs_dlg_execute(void)
{
     gint ret;

     prefs_dlg_build();
     ret = gnome_dialog_run(GNOME_DIALOG(prefs_dlg.dialog));

     switch (ret) {
     case 0:		/* OK */
	  prefs_dlg_main();
     case 1:		/* Cancel */
	  gnome_dialog_close(GNOME_DIALOG(prefs_dlg.dialog));
     case -1:		/* Esc or "closed window" */
	  break;
     default:
	  g_assert_not_reached();
     }
	  
     return ret == 0; /* Ok */
}






