#include <config.h>

#include <ctype.h>
#include <langinfo.h>
#include <stdio.h>
#include <time.h>
#include <unistd.h>

#include <Python.h>
#include <compile.h>  /* why is python-dev designed so, that I have      */
#include <eval.h>     /* to implicitily include compile.h and eval.h ??? */

#include <gnome.h>

#include "menu.h"
#include "misc.h"

#include "py_module.h"


PyObject *py_module_globals;

PyObject *py_module_amount(PyObject *self, PyObject *args);
PyObject *py_module_from(PyObject *self, PyObject *args);
PyObject *py_module_to(PyObject *self, PyObject *args);
PyObject *py_module_date(PyObject *self, PyObject *args);
PyObject *py_module_day(PyObject *self, PyObject *args);
PyObject *py_module_weekday(PyObject *self, PyObject *args);
PyObject *py_module_month(PyObject *self, PyObject *args);
PyObject *py_module_year(PyObject *self, PyObject *args);
PyObject *py_module_desc(PyObject *self, PyObject *args);
PyObject *py_module_parents(PyObject *self, PyObject *args);

PyObject *py_module_register_func(PyObject *self, PyObject *args);
PyObject *py_module_register_report(PyObject *self, PyObject *args);

PyObject *py_module_date_value(PyObject *self, PyObject *args);

static PyMethodDef EmmaMethods[] = {
     { "amount", py_module_amount, METH_VARARGS },
     { "acc_from", py_module_from, METH_VARARGS },
     { "acc_to", py_module_to, METH_VARARGS },
     { "date", py_module_date, METH_VARARGS },
     { "day", py_module_day, METH_VARARGS },
     { "weekday", py_module_weekday, METH_VARARGS },
     { "month", py_module_month, METH_VARARGS },
     { "year", py_module_year, METH_VARARGS },
     { "desc", py_module_desc, METH_VARARGS },
     { "parents", py_module_parents, METH_VARARGS },

     { "register_func", py_module_register_func, METH_VARARGS },
     { "register_report", py_module_register_report, METH_VARARGS },

     { "date_val", py_module_date_value, METH_VARARGS },

     { NULL, NULL }
};

void py_module_catch_python_error(void);

void py_module_import_module(const gchar *module_name)
{
     gchar cmd[1024] = "import ";
     strcat(cmd, module_name);
     PyRun_String(cmd, Py_single_input, py_module_globals, NULL);
     if (PyErr_Occurred()) {
	  py_module_catch_python_error();
	  g_printerr(_("An error occurred while trying to import %s module.\n"),
		     module_name);
	  g_printerr("%s", py_module_get_error_string());
	  py_module_clear_error();
     }
}

void py_module_include_file(gchar *filename, const gboolean verbose_cant_find)
{
     if (!access(filename, F_OK)) { 	/* file exist ? */
	  FILE *stream;

	  stream = fopen(filename, "r");
	  PyRun_File(stream, filename, Py_file_input, py_module_globals, NULL);
	  fclose(stream);

	  if (PyErr_Occurred()) {
	    py_module_catch_python_error();
	    g_printerr(_("An error occurred while trying to parse file %s.\n"),
		       filename);
	    g_printerr("%s", py_module_get_error_string());
	    py_module_clear_error();
	  }
     } else
	  if (verbose_cant_find)
	       g_printerr("Coudn't access python source-file %s\n", filename);
}

void py_module_include_appriopriate_files(void)
{
     gchar tmp[1024];

     strcpy(tmp, g_get_home_dir());
     strcat(tmp, "/.emma.py");
     py_module_include_file(tmp, FALSE);

     py_module_include_file("/usr/share/emma.py", FALSE);
     py_module_include_file("/usr/local/share/emma.py", FALSE);

     py_module_include_file("/usr/share/emma/emma.py", FALSE);
     py_module_include_file("/usr/local/share/emma/emma.py", FALSE);
}

void py_module_redirect_stderr(void)
{
     PyObject *mod, *klass, *obj;

     mod = PyImport_ImportModule("StringIO");
     klass = PyObject_GetAttrString(mod, "StringIO");

     obj = PyObject_CallFunction(klass, NULL);
     Py_DECREF(klass);
     
     PySys_SetObject("stderr", obj);
     Py_DECREF(obj);
}

void py_module_init(void)
{
     py_module_func_desc = NULL;

     Py_Initialize();

     py_module_globals = PyDict_New();
     PyDict_SetItemString(py_module_globals, "__builtins__", PyEval_GetBuiltins());

     py_module_redirect_stderr();

     PyImport_AddModule("emma");
     Py_InitModule("emma", EmmaMethods);
     PyRun_String("import emma", Py_single_input, py_module_globals, NULL);

     py_module_include_appriopriate_files();
}

void py_module_done(void)
{
     GList *cursor;

     for (cursor = py_module_func_desc; cursor != NULL; cursor = g_list_next(cursor)) {
	  PyModuleFuncDesc *func_desc = PyModuleFuncDescPtr(cursor->data);
	  g_free(func_desc->name);
	  g_free(func_desc->desc);
     }
     g_list_free(py_module_func_desc);

     Py_DECREF(py_module_globals);
     Py_Finalize();
}


/***********************************************
 *   One function to eval amounts (numbers)    *
 ***********************************************/

gdouble py_module_eval_amount(gchar *expr)
{
     PyObject *result;
     gchar *tmp;
     gchar *old_locale;

     /* FIXME: this locale switching is just to work around
	the bug with ignoring fractional part of numbers
	in some locales.
	I wonder if it should be done in py_module_eval_limit.
	I think it has something to do with python library.
     */
     old_locale = g_strdup(setlocale(LC_NUMERIC, NULL));
     setlocale(LC_NUMERIC, "C");

     tmp = g_strdup_printf("float((%s))", expr);
     result = PyRun_String(tmp, Py_eval_input, py_module_globals, NULL);
     g_free(tmp);

     /*FIXME: here the locale is restored (look above)*/
     setlocale(LC_NUMERIC, old_locale);
     g_free(old_locale);

     if (result == NULL) {
	  py_module_catch_python_error();
	  return 0;
     } else {
	  gdouble tmp = PyFloat_AsDouble(result);
	  Py_DECREF(result);
	  return tmp;
     }
}

/***********************************************
 *  Four functions to eval transactions limits *
 ***********************************************/

PyObject *py_module_compiled_limit;
TransactionData *py_module_transaction;

void py_module_compile_limit(const gchar *expr)
{
     gchar expr_str[PY_EXPRESSION_MAX_SIZE];

     sprintf(expr_str, "%s", only_white_space(expr) ? "1" : expr);

     py_module_compiled_limit = Py_CompileString(expr_str, "<stdin>", Py_eval_input);
     if (py_module_compiled_limit == NULL)
	  py_module_catch_python_error();
}

void py_module_release_compiled_limit(void)
{
     g_assert(py_module_compiled_limit != NULL);

     Py_DECREF(py_module_compiled_limit);
     py_module_compiled_limit = NULL;
     
     py_module_transaction = NULL;
}

gboolean py_module_eval_limit(TransactionData *trans)
{
     PyObject *result;

     g_assert(py_module_compiled_limit != NULL);

     py_module_transaction = trans;
     result = PyEval_EvalCode((PyCodeObject *)py_module_compiled_limit, py_module_globals, NULL);
     if (result == NULL) {
	  py_module_catch_python_error();
	  return py_module_transaction->inbound;
     } else {
	  gboolean tmp = PyInt_AsLong(result);
	  Py_DECREF(result);
	  return tmp;
     }
}

/***********************************************
 *     Functions that deal with errors         *
 ***********************************************/

gchar *py_module_error_string = NULL;

void py_module_catch_python_error(void)
{
     PyObject *obj, *str;

     g_assert(py_module_error_string == NULL); /* no previous, unhandled errors ? */
     g_assert(PyErr_Occurred());

     PyErr_Print();

     obj = PySys_GetObject("stderr");
     str = PyObject_CallMethod(obj, "getvalue", NULL);
     py_module_error_string = g_strdup_printf(PyString_AsString(str));
     Py_DECREF(str);
     /* don't do it !!! - Py_DECREF(obj); */
}

gboolean py_module_error_occurred(void)
{
     return py_module_error_string != NULL;
}

void py_module_clear_error(void)
{
     PyObject *obj, *str;

     g_assert(py_module_error_string != NULL); /* there has been an error before */

     obj = PySys_GetObject("stderr");
     str = PyString_FromString("");
     PyObject_SetAttrString(obj, "buf", str);
     Py_DECREF(str);

     g_free(py_module_error_string);
     py_module_error_string = NULL;
}

const gchar *py_module_get_error_string(void)
{
     g_assert(py_module_error_string != NULL); /* there has been an error before */

     return py_module_error_string;
}

/***********************************************
 *    Functions exported in emma module        *
 ***********************************************/

PyObject *py_module_amount(PyObject *self, PyObject *args)
{
     if (py_module_transaction == NULL) {
	  gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), "emma.amount()");
	  PyErr_SetString(PyExc_RuntimeError, text);
	  g_free(text);
	  return NULL;
     } else 
	  return Py_BuildValue("d",(double)(py_module_transaction->amount));
}

PyObject *py_module_from(PyObject *self, PyObject *args)
{
     if (py_module_transaction == NULL) {
	  gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), "emma.acc_from()");
	  PyErr_SetString(PyExc_RuntimeError, text);
	  g_free(text);
	  return NULL;
     } else 
	  return Py_BuildValue("s",py_module_transaction->acc_from->name);
}

PyObject *py_module_to(PyObject *self, PyObject *args)
{
     if (py_module_transaction == NULL) {
	  gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), "emma.acc_to()");
	  PyErr_SetString(PyExc_RuntimeError, text);
	  g_free(text);
	  return NULL;
     } else 
	  return Py_BuildValue("s",py_module_transaction->acc_to->name);
}

PyObject *py_module_date(PyObject *self, PyObject *args)
{
     if (py_module_transaction == NULL) {
	  gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), "emma.date()");
	  PyErr_SetString(PyExc_RuntimeError, text);
	  g_free(text);
	  return NULL;
     } else {
	  struct tm tmp;
	  memset(&tmp, '\0', sizeof(struct tm));
	  /* FIXME - why the commented line isn't working? 
	   * (it messes with fields of struct tm other than year,mon,mday)
	   * 3 lines below are a quick workaround of this
	  g_date_to_struct_tm(&(py_module_transaction->date), &tmp);
	  */
	  tmp.tm_mday = g_date_day(&(py_module_transaction->date));
	  tmp.tm_mon = g_date_month(&(py_module_transaction->date)) - 1;
	  tmp.tm_year = g_date_year(&(py_module_transaction->date)) - 1900;
	  
	  return Py_BuildValue("d",(double)mktime(&tmp));
     }
}

PyObject *py_module_day(PyObject *self, PyObject *args)
{
  if (py_module_transaction == NULL) {
    gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"),
				  "emma.day()");
    PyErr_SetString(PyExc_RuntimeError, text);
    g_free(text);
    return NULL;
  } else 
    return Py_BuildValue("i", g_date_day(&(py_module_transaction->date)));
}

PyObject *py_module_weekday(PyObject *self, PyObject *args)
{
  if (py_module_transaction == NULL) {
    gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), 
				  "emma.day()");
    PyErr_SetString(PyExc_RuntimeError, text);
    g_free(text);
    return NULL;
  } else 
    return Py_BuildValue("i", g_date_weekday(&(py_module_transaction->date)));
}

PyObject *py_module_month(PyObject *self, PyObject *args)
{
  if (py_module_transaction == NULL) {
    gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), 
				  "emma.month()");
    PyErr_SetString(PyExc_RuntimeError, text);
    g_free(text);
    return NULL;
  } else 
    return Py_BuildValue("i", g_date_month(&(py_module_transaction->date)));
}

PyObject *py_module_year(PyObject *self, PyObject *args)
{
  if (py_module_transaction == NULL) {
    gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), 
				  "emma.year()");
    PyErr_SetString(PyExc_RuntimeError, text);
    g_free(text);
    return NULL;
  } else 
    return Py_BuildValue("i", g_date_year(&(py_module_transaction->date)));
}

PyObject *py_module_desc(PyObject *self, PyObject *args)
{
     if (py_module_transaction == NULL) {
	  gchar *text = g_strdup_printf(_("Use of %s allowed only in limits"), "emma.desc()");
	  PyErr_SetString(PyExc_RuntimeError, text);
	  g_free(text);
	  return NULL;
     } else 
	  return Py_BuildValue("s",py_module_transaction->desc);
}

PyObject *py_module_parents(PyObject *self, PyObject *args)
{
     PyObject *res = NULL;
     gchar *acc_name;

     if (PyArg_ParseTuple(args, "s:parents", &acc_name)) {
	  GList *cursor;
	  AccountData *parent = NULL;

	  for (cursor = acc_list; cursor != NULL; cursor = g_list_next(cursor)) 
	       if (strcmp(acc_name, AccountPtr(cursor->data)->name) == 0) {
		    parent = AccountPtr(cursor->data)->parent;
		    break;
	       }		

	  res = PyList_New(0);
	  while (parent != NULL) {
	       PyList_Append(res, Py_BuildValue("s",parent->name));
	       parent = parent->parent;
	  }
     }
     
     return res;
}

PyObject *py_module_date_value(PyObject *self, PyObject *args)
{
     PyObject *date_val = NULL;
     gchar *date_str;

     if (PyArg_ParseTuple(args, "s:date_val", &date_str)) {
	  GDate date;
	  struct tm tmp;

	  memset(&tmp, '\0', sizeof(struct tm));
	  strptime(date_str, misc_date_fmt, &tmp);
	  /*
	  g_date_set_dmy(&date, tmp.tm_mday, tmp.tm_mon+1, tmp.tm_year+1900);
	  g_date_to_struct_tm(&date, &tmp);
	  */
	  date_val = Py_BuildValue("d",(double)mktime(&tmp));
     }

     return date_val;
}

PyObject *py_module_register_func(PyObject *self, PyObject *args)
{
     PyObject *value = NULL;
     gchar *name, *desc;

     if (PyArg_ParseTuple(args, "ss:register_func", &name, &desc)) {
	  PyModuleFuncDesc *new_func_desc;
	  
	  new_func_desc = g_new(PyModuleFuncDesc, 1);
	  new_func_desc->name = g_strdup(name);
	  new_func_desc->desc = g_strdup(desc);
	  py_module_func_desc = g_list_prepend(py_module_func_desc, new_func_desc);

	  Py_INCREF(Py_None);
	  value = Py_None;
     }

     return value;
}

PyObject *py_module_register_report(PyObject *self, PyObject *args)
{
     PyObject *value = NULL;
     gchar *command, *name, *group = NULL;

     if (PyArg_ParseTuple(args, "ss|z:register_func", &command, &name, &group)) {
	  menu_add_report_filter(_(command), _(name), _(group));

	  Py_INCREF(Py_None);
	  value = Py_None;
     }

     return value;
}

