#include <config.h>

#include "data.h"
#include "interface.h" /* just to get interface.application */
#include "misc.h"

#include "report_acc_axis.h"

static struct _Dialog {
  GtkWidget *dialog;

  GtkWidget *tree;
  GtkWidget *list;

  GtkWidget *left, *right;
  GtkWidget *move_up, *move_down;
  GtkWidget *reverse;
} dialog;

void report_acc_axis_init(void)
{
  report_acc_axis_list = NULL;
}

void report_acc_axis_done(void)
{
  GList *cursor;

  for (cursor = report_acc_axis_list; 
       cursor != NULL; 
       cursor = g_list_next(cursor))
    g_free(ReportAccountDataPtr(cursor->data));

  g_list_free(report_acc_axis_list);
}

/* This function removes dangling accounts pointers from report_acc_axis_list
 * (it is not *the right way*, but it works so no f-ixme here
 */
void report_acc_axis_check(void)
{
  GList *cursor, *new_list;

  new_list = NULL;
  for (cursor = report_acc_axis_list; 
       cursor != NULL; 
       cursor = g_list_next(cursor)) {
    ReportAccountData *rad = ReportAccountDataPtr(cursor->data);
    if (g_list_find(acc_list, rad->account) != NULL)
      new_list = g_list_prepend(new_list, rad);
  }
  g_list_free(report_acc_axis_list);
  report_acc_axis_list = g_list_reverse(new_list);
}

/* adds appropriate row to the clist  on the right-hand-side */
static const gint ROW_UNKNOWN = -1;
static void dialog_setup_rad(ReportAccountData *rad, gboolean select, gint row)
{
  gchar *text[2];

  if (row == ROW_UNKNOWN) {
    text[0] = rad->account->name;
    text[1] = rad->reversed ? _("rev") : "";

    row = gtk_clist_append(GTK_CLIST(dialog.list), text);
    gtk_clist_set_row_data_full(GTK_CLIST(dialog.list), row, rad, g_free);
  } else {
    gtk_clist_set_text(GTK_CLIST(dialog.list), row, 0, rad->account->name);
    gtk_clist_set_text(GTK_CLIST(dialog.list), row, 1, 
		       rad->reversed ? _("rev") : "");
  }

  if (select)
    gtk_clist_select_row(GTK_CLIST(dialog.list), row, 0);
}

/* callback for pressing a button in ctree on the left of the dialog */
static gint dialog_acc_tree_press_event(GtkWidget *ctree, 
					GdkEventButton *event, 
					gpointer *data)
{
     gint row, column;
     if (!event) {
          return FALSE;
     }
    
     if (!gtk_clist_get_selection_info(GTK_CLIST(ctree), event->x, event->y,
 				       &row, &column)) {
	  return FALSE;
     }
 
     if (event->button == 3) {
	/* ... */
	GtkCTreeNode *node;
	node = gtk_ctree_node_nth(GTK_CTREE(ctree), row);

	gtk_clist_freeze(GTK_CLIST(ctree));

	gtk_ctree_expand_to_depth(GTK_CTREE(ctree), node, 1);
	gtk_ctree_post_recursive_to_depth(GTK_CTREE(ctree), node, 2,
				  GTK_CTREE_FUNC(gtk_ctree_select), NULL);
	gtk_ctree_unselect(GTK_CTREE(ctree), node);

	gtk_clist_thaw(GTK_CLIST(ctree));

	return TRUE;
     }
    
     return FALSE;   
}
 
/* callback function for dialog.left widget */
static void dialog_left(GtkWidget *widget, gpointer callback_data)
{
  GList *selection;
  gint row_to_remove;

  gtk_clist_freeze(GTK_CLIST(dialog.list));

  /* mark rows to delete with NULL data associated */
  for (selection = GTK_CLIST(dialog.list)->selection;
       selection != NULL;
       selection = g_list_next(selection)) {
    
    ReportAccountData *rad;
    gint row;

    row = GPOINTER_TO_INT(selection->data);
    rad = gtk_clist_get_row_data(GTK_CLIST(dialog.list), row); 
    gtk_clist_set_row_data(GTK_CLIST(dialog.list), row, NULL);
  }

  /* remove rows with data=NULL */
  for (row_to_remove = 0; 
       row_to_remove < GTK_CLIST(dialog.list)->rows; 
       row_to_remove++) 
 
    if (gtk_clist_get_row_data(GTK_CLIST(dialog.list), row_to_remove) == NULL){
      gtk_clist_remove(GTK_CLIST(dialog.list), 
		       row_to_remove);
      row_to_remove--;
    }

  gtk_clist_thaw(GTK_CLIST(dialog.list));
}

/* callback function for dialog.right widget */
static void dialog_right(GtkWidget *widget, gpointer callback_data)
{
  GList *selection;

  /* move the selection to the right-hand-side list */
  gtk_clist_freeze(GTK_CLIST(dialog.list));
  gtk_clist_unselect_all(GTK_CLIST(dialog.list));
  for (selection = g_list_first(GTK_CLIST(dialog.tree)->selection);
       selection != NULL;
       selection = g_list_next(selection)) {

    AccountData *account;
    ReportAccountData *rad;

    account = gtk_ctree_node_get_row_data(GTK_CTREE(dialog.tree), 
					  selection->data);

    rad = g_new(ReportAccountData, 1);
    rad->account = account;
    rad->reversed = account->reversed;

    dialog_setup_rad(rad, TRUE, ROW_UNKNOWN);
  }
  gtk_clist_thaw(GTK_CLIST(dialog.list));

  /* unselect items in the left-hand-side tree */
  gtk_ctree_unselect_recursive(GTK_CTREE(dialog.tree), NULL);
}

/* this function is used as a compare function in two callbacks below */
static gint dialog_compare_ints(gconstpointer a, gconstpointer b) {
  return (GPOINTER_TO_INT(a) - GPOINTER_TO_INT(b));
  /* [lanfor] - I think no range checking needs to be done - the
     numbers a and b should be small enough not to cause overflow */
}

/* callback function for dialog.move_up widget */
static void dialog_move_up(GtkWidget *widget, gpointer callback_data)
{
  GList *selection, *cursor;
  gint row_from, row_to;

  selection = g_list_copy(GTK_CLIST(dialog.list)->selection);
  selection = g_list_sort(selection, dialog_compare_ints);
  g_return_if_fail(selection);

  row_to = GPOINTER_TO_INT(g_list_last(selection)->data);

  row_from = GPOINTER_TO_INT(g_list_first(selection)->data) - 1;
  if (row_from < 0)
    row_from = row_to;

  for (cursor = g_list_last(selection); 
       cursor->prev != NULL; 
       cursor = cursor->prev /* no g_list_prev function ??? */ ) {
    
    gint current_row, prev_row;
    current_row = GPOINTER_TO_INT(cursor->data);
    prev_row = GPOINTER_TO_INT(cursor->prev->data);
    
    if ((current_row - 1) != prev_row) {
      row_from = current_row - 1;
      break;
    }
  }
  gtk_clist_row_move(GTK_CLIST(dialog.list), row_from, row_to);
  g_list_free(selection);
}

/* callback function for dialog.move_down widget */
static void dialog_move_down(GtkWidget *widget, gpointer callback_data)
{
  GList *selection, *cursor;
  gint row_from, row_to;

  selection = g_list_copy(GTK_CLIST(dialog.list)->selection);
  selection = g_list_sort(selection, dialog_compare_ints);
  g_return_if_fail(selection);

  row_to = GPOINTER_TO_INT(g_list_first(selection)->data);

  row_from = GPOINTER_TO_INT(g_list_last(selection)->data) + 1;
  if (row_from >= (GTK_CLIST(dialog.list)->rows))
    row_from = row_to;

  for (cursor = g_list_first(selection); 
       cursor->next != NULL; 
       cursor = g_list_next(cursor)) {

    gint current_row, next_row;
    current_row = GPOINTER_TO_INT(cursor->data);
    next_row = GPOINTER_TO_INT(cursor->next->data);
    
    if ((current_row + 1) != next_row) {
      row_from = current_row + 1;
      break;
    }
  }
  gtk_clist_row_move(GTK_CLIST(dialog.list), row_from, row_to);
  g_list_free(selection);
}

/* callback function for dialog.reverse widget */
static void dialog_reverse(GtkWidget *widget, gpointer callback_data)
{
  GList *selection;

  g_return_if_fail(GTK_CLIST(dialog.list)->selection != NULL);

  gtk_clist_freeze(GTK_CLIST(dialog.list));

  for (selection = GTK_CLIST(dialog.list)->selection;
       selection != NULL;
       selection = g_list_next(selection)) {
    
    ReportAccountData *rad;
    gint row;

    row = GPOINTER_TO_INT(selection->data);
    rad = gtk_clist_get_row_data(GTK_CLIST(dialog.list), row); 

    rad->reversed = !(rad->reversed);

    dialog_setup_rad(rad, TRUE, row);
  }
  gtk_clist_thaw(GTK_CLIST(dialog.list));
}

/* builds the overall structure of the dialog widget */
static void dialog_build(void)
{
   GtkWidget *scrolled_window;
   GtkWidget *table;
   GtkWidget *label;

   /* build the dialog window */
   dialog.dialog = gnome_dialog_new(_("Edit accounts axis..."), 
				    GNOME_STOCK_BUTTON_OK, 
				    GNOME_STOCK_BUTTON_CANCEL, NULL);
   gnome_dialog_set_parent(GNOME_DIALOG(dialog.dialog), 
			   GTK_WINDOW(interface.application));
   gnome_dialog_set_default(GNOME_DIALOG(dialog.dialog), 0);

   table = gtk_table_new(3, 8, FALSE);

   /* the left side of the dialog - accounts to choose from */
   gtk_widget_show(label = gtk_label_new(_("Accounts to choose from:")));
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

   dialog.tree = misc_build_ctree(acc_list, NULL);
   gtk_clist_set_selection_mode(GTK_CLIST(dialog.tree), GTK_SELECTION_MULTIPLE);
   gtk_signal_connect(GTK_OBJECT(dialog.tree), "button_press_event",
		      GTK_SIGNAL_FUNC(dialog_acc_tree_press_event), NULL);
   gtk_table_attach(GTK_TABLE(table), dialog.tree, 
		    0, 1, 1, 8, 
		    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 3, 3);
  
   /* the middle - buttons */
   dialog.left = gtk_button_new_with_label(_("<--"));
   gtk_widget_show(dialog.left);
   gtk_table_attach(GTK_TABLE(table), dialog.left, 
		    1, 2, 1, 2, 0, 0, 3, 3);
   gtk_signal_connect(GTK_OBJECT(dialog.left), "clicked", 
		      GTK_SIGNAL_FUNC(dialog_left), NULL);
  
   dialog.right = gtk_button_new_with_label(_("-->"));
   gtk_widget_show(dialog.right);
   gtk_table_attach(GTK_TABLE(table), dialog.right, 
		    1, 2, 2, 3, 0, 0, 3, 3);
   gtk_signal_connect(GTK_OBJECT(dialog.right), "clicked", 
		      GTK_SIGNAL_FUNC(dialog_right), NULL);
  
   gtk_widget_show(label = gtk_hseparator_new());
   gtk_table_attach(GTK_TABLE(table), label, 1, 2, 3, 4, GTK_FILL, 0, 3, 3);
  
   dialog.move_up = gtk_button_new_with_label(_("Up"));
   gtk_widget_show(dialog.move_up);
   gtk_table_attach(GTK_TABLE(table), dialog.move_up, 
		    1, 2, 4, 5, 0, 0, 3, 3);
   gtk_signal_connect(GTK_OBJECT(dialog.move_up), "clicked", 
		      GTK_SIGNAL_FUNC(dialog_move_up), NULL);
  
   dialog.move_down = gtk_button_new_with_label(_("Down"));
   gtk_widget_show(dialog.move_down);
   gtk_table_attach(GTK_TABLE(table), dialog.move_down, 
		    1, 2, 5, 6, 0, 0, 3, 3);
   gtk_signal_connect(GTK_OBJECT(dialog.move_down), "clicked", 
		      GTK_SIGNAL_FUNC(dialog_move_down), NULL);

   gtk_widget_show(label = gtk_hseparator_new());
   gtk_table_attach(GTK_TABLE(table), label, 1, 2, 6, 7, GTK_FILL, 0, 3, 3);
  
   dialog.reverse = gtk_button_new_with_label(_("Rev"));
   gtk_widget_show(dialog.reverse);
   gtk_table_attach(GTK_TABLE(table), dialog.reverse, 
		    1, 2, 7, 8, 0, 0, 3, 3);
   gtk_signal_connect(GTK_OBJECT(dialog.reverse), "clicked", 
		      GTK_SIGNAL_FUNC(dialog_reverse), NULL);
  
   /* the right side of the dialog - the list of choosen accounts with flags */
   gtk_widget_show(label = gtk_label_new(_("Accounts axis:")));
   gtk_table_attach(GTK_TABLE(table), label, 2, 3, 0, 1, GTK_FILL, 0, 3, 3);
   
   dialog.list = gtk_clist_new(2);
   gtk_clist_column_titles_hide(GTK_CLIST(dialog.list));
   gtk_clist_set_column_width(GTK_CLIST(dialog.list), 0, 150);
   gtk_clist_set_selection_mode(GTK_CLIST(dialog.list), GTK_SELECTION_MULTIPLE);
   gtk_widget_set_usize(dialog.list, 200, 200);
   gtk_widget_show(dialog.list);
   
   scrolled_window = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add(GTK_CONTAINER(scrolled_window), dialog.list);
   gtk_widget_show(scrolled_window);

   gtk_table_attach(GTK_TABLE(table), scrolled_window, 
		    2, 3, 1, 8, 
		    GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 3, 3);

   /* pack everything up into the dialog */
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog.dialog)->vbox), 
		      table, TRUE, TRUE, 0);
   gtk_widget_show(table);
}

/* this function presets widgets in the dialog according to what is
   held in report_acc_axis_list (it is mainly responsible for setting
   up right-hand-side clist with appropriate data associated with the rows */
static void dialog_preset_values(void)
{
  GList *cursor;

  for (cursor = report_acc_axis_list; 
       cursor != NULL; 
       cursor = g_list_next(cursor)) {

    ReportAccountData *rad;

    rad = g_new(ReportAccountData, 1);
    memcpy(rad, ReportAccountDataPtr(cursor->data), sizeof(ReportAccountData));

    dialog_setup_rad(rad, FALSE, ROW_UNKNOWN);
  }
}

/* dialog_mains syncs the dialog contents with the report_acc_axis_list */
static void dialog_main(void)
{
  gint row;
  
  report_acc_axis_done(); /* free report_acc_axis_list */
  report_acc_axis_list = NULL;

  for (row = 0; row < GTK_CLIST(dialog.list)->rows; row++) {
    ReportAccountData *rad, *orig;
    orig = ReportAccountDataPtr(
	           gtk_clist_get_row_data(GTK_CLIST(dialog.list), row));
    rad = g_new(ReportAccountData, 1);
    memcpy(rad, orig, sizeof(ReportAccountData));

    report_acc_axis_list = g_list_prepend(report_acc_axis_list, rad);
    gtk_clist_set_row_data(GTK_CLIST(dialog.list), row, NULL);
  }
  report_acc_axis_list = g_list_reverse(report_acc_axis_list);
}

gboolean report_acc_axis_edit(void)
{
  gint ret;
  
  dialog_build();
  dialog_preset_values();

  ret = gnome_dialog_run(GNOME_DIALOG(dialog.dialog));
  switch (ret) {
  case 0:		/* OK */
    dialog_main();
  case 1:		/* Cancel */
    gnome_dialog_close(GNOME_DIALOG(dialog.dialog));
  case -1:		/* Esc or "closed window" */
    break;
  default:
    g_assert_not_reached();
  }

  return ret == 0; /* Ok */
}

