#include <config.h>

#include "interface.h" /* just to get interface.application */
#include "limit_dlg.h"

#include "report_grp_axis.h"

static struct _Dialog {
  GtkWidget *dialog;

  GtkWidget *list;

  GtkWidget *label, *expr;

  GtkWidget *add_group, *edit_group, *remove_group;
  GtkWidget *move_up, *move_down;
  GtkWidget *edit_expr;
} dialog;

void report_grp_axis_init(void)
{
  ReportGroupData *rgd;
  
  rgd = g_new(ReportGroupData, 1);
  strcpy(rgd->label, "");
  strcpy(rgd->expr, "1");

  report_grp_axis_list = NULL;
  report_grp_axis_list = g_list_append(report_grp_axis_list, rgd);
}

void report_grp_axis_done(void)
{
  GList *cursor;
  
  for (cursor = report_grp_axis_list; 
       cursor != NULL; 
       cursor = g_list_next(cursor))
    g_free(ReportGroupDataPtr(cursor->data));

  g_list_free(report_grp_axis_list);
}

/* adds appropriate row to the clist  on the right-hand-side */
static void dialog_setup_rgd(ReportGroupData *rgd, gboolean select)
{
  gint row;

  row = gtk_clist_find_row_from_data(GTK_CLIST(dialog.list), rgd);
  if (row == (-1)) {
    gchar *text[2];

    text[0] = rgd->label;
    text[1] = rgd->expr;
  
    row = gtk_clist_append(GTK_CLIST(dialog.list), text);
    gtk_clist_set_row_data_full(GTK_CLIST(dialog.list), row, rgd, g_free);
  } else {
    ReportGroupData *tmp;
    tmp = g_new(ReportGroupData, 1);
    memcpy(tmp, rgd, sizeof(ReportGroupData));

    gtk_clist_set_text(GTK_CLIST(dialog.list), row, 0, tmp->label);
    gtk_clist_set_text(GTK_CLIST(dialog.list), row, 1, tmp->expr);
    gtk_clist_set_row_data_full(GTK_CLIST(dialog.list), row, tmp, g_free);
  }

  if (select)
    gtk_clist_select_row(GTK_CLIST(dialog.list), row, 0);
}

/* callback function for dialog.edit_expr widget */
static void dialog_edit_expr(GtkWidget *widget, gpointer callback_data)
{
  gchar expr[PY_EXPRESSION_MAX_SIZE];
  strcpy(expr, gtk_entry_get_text(GTK_ENTRY(dialog.expr)));

  if (limit_dlg_execute(_("Edit group limit..."), expr))
    gtk_entry_set_text(GTK_ENTRY(dialog.expr), expr);
}

/* callback function for dialog.add_group widget */
static void dialog_add_group(GtkWidget *widget, gpointer callback_data)
{
  ReportGroupData *rgd;

  rgd = g_new(ReportGroupData, 1);
  strcpy(rgd->label, gtk_entry_get_text(GTK_ENTRY(dialog.label)));
  strcpy(rgd->expr, gtk_entry_get_text(GTK_ENTRY(dialog.expr)));

  dialog_setup_rgd(rgd, TRUE);
}

/* callback function for dialog.edit_group widget */
static void dialog_edit_group(GtkWidget *widget, gpointer callback_data)
{
  ReportGroupData *rgd;
  gint row;

  g_return_if_fail(GTK_CLIST(dialog.list)->selection != NULL);
  row = GPOINTER_TO_INT(GTK_CLIST(dialog.list)->selection->data);
  
  rgd = gtk_clist_get_row_data(GTK_CLIST(dialog.list), row);
  strcpy(rgd->label, gtk_entry_get_text(GTK_ENTRY(dialog.label)));
  strcpy(rgd->expr, gtk_entry_get_text(GTK_ENTRY(dialog.expr)));

  dialog_setup_rgd(rgd, TRUE);
}

/* callback function for dialog.remove_group widget */
static void dialog_remove_group(GtkWidget *widget, gpointer callback_data)
{
  gint row;

  g_return_if_fail(GTK_CLIST(dialog.list)->selection != NULL);
  row = GPOINTER_TO_INT(GTK_CLIST(dialog.list)->selection->data);

  gtk_clist_remove(GTK_CLIST(dialog.list), row);
}

/* callback function for dialog.move_up widget */
static void dialog_move_up(GtkWidget *widget, gpointer callback_data)
{
  gint row;

  g_return_if_fail(GTK_CLIST(dialog.list)->selection != NULL);
  row = GPOINTER_TO_INT(GTK_CLIST(dialog.list)->selection->data);
  if (row != 0)
    gtk_clist_swap_rows(GTK_CLIST(dialog.list), row, row-1);
}

/* callback function for dialog.move_down widget */
static void dialog_move_down(GtkWidget *widget, gpointer callback_data)
{
  gint row;

  g_return_if_fail(GTK_CLIST(dialog.list)->selection != NULL);
  row = GPOINTER_TO_INT(GTK_CLIST(dialog.list)->selection->data);
  if (row != GTK_CLIST(dialog.list)->rows)
    gtk_clist_swap_rows(GTK_CLIST(dialog.list), row, row+1);
}

/* callback function triggered on row selection */
void dialog_select_row(GtkCList *clist, gint row, gint column,
		       GdkEventButton *event, gpointer user_data)
{
  ReportGroupData *rgd = gtk_clist_get_row_data(clist, row);
  gtk_entry_set_text(GTK_ENTRY(dialog.label), rgd->label);
  gtk_entry_set_text(GTK_ENTRY(dialog.expr), rgd->expr);
}


/* builds the overall structure of the dialog widget */
static void dialog_build(void)
{
  GtkWidget *scrolled_window;
  GtkWidget *table;
  GtkWidget *label;

  /* build the dialog window */
  dialog.dialog = gnome_dialog_new(_("Edit groups axis..."), 
				   GNOME_STOCK_BUTTON_OK, 
				   GNOME_STOCK_BUTTON_CANCEL, NULL);
  gnome_dialog_set_parent(GNOME_DIALOG(dialog.dialog), 
			  GTK_WINDOW(interface.application));
  gnome_dialog_set_default(GNOME_DIALOG(dialog.dialog), 0);

  table = gtk_table_new(6, 5, FALSE);

  /* The widgets for label & expression entering */
  gtk_widget_show(label = gtk_label_new(_("Label:")));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

  dialog.label = gtk_entry_new();
  gtk_widget_show(dialog.label);
  gtk_table_attach(GTK_TABLE(table), dialog.label, 
		   1, 6, 0, 1, GTK_FILL | GTK_EXPAND, 0, 3, 3);

  gtk_widget_show(label = gtk_label_new(_("Expr:")));
  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, GTK_FILL, 0, 3, 3);

  dialog.expr = gtk_entry_new();
  gtk_widget_show(dialog.expr);
  gtk_table_attach(GTK_TABLE(table), dialog.expr, 
		   1, 5, 1, 2, GTK_FILL | GTK_EXPAND, 0, 3, 3);

  dialog.edit_expr = gtk_button_new_with_label(_("Edit..."));
  gtk_widget_show(dialog.edit_expr);
  gtk_table_attach(GTK_TABLE(table), dialog.edit_expr, 
		   5, 6, 1, 2, GTK_FILL, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.edit_expr), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_edit_expr), NULL);

  /* Buttons for manipulating the list of groups */
  dialog.add_group = gtk_button_new_with_label(_("Add"));
  gtk_widget_show(dialog.add_group);
  gtk_table_attach(GTK_TABLE(table), dialog.add_group, 
		   0, 1, 2, 3, 0, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.add_group), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_add_group), NULL);

  dialog.edit_group = gtk_button_new_with_label(_("Edit"));
  gtk_widget_show(dialog.edit_group);
  gtk_table_attach(GTK_TABLE(table), dialog.edit_group, 
		   1, 2, 2, 3, 0, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.edit_group), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_edit_group), NULL);

  dialog.remove_group = gtk_button_new_with_label(_("Delete"));
  gtk_widget_show(dialog.remove_group);
  gtk_table_attach(GTK_TABLE(table), dialog.remove_group, 
		   2, 3, 2, 3, 0, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.remove_group), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_remove_group), NULL);

  dialog.move_up = gtk_button_new_with_label(_("Up"));
  gtk_widget_show(dialog.move_up);
  gtk_table_attach(GTK_TABLE(table), dialog.move_up, 
		   4, 5, 2, 3, 0, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.move_up), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_move_up), NULL);

  dialog.move_down = gtk_button_new_with_label(_("Down"));
  gtk_widget_show(dialog.move_down);
  gtk_table_attach(GTK_TABLE(table), dialog.move_down, 
		   5, 6, 2, 3, 0, 0, 3, 3);
  gtk_signal_connect(GTK_OBJECT(dialog.move_down), "clicked", 
		     GTK_SIGNAL_FUNC(dialog_move_down), NULL);

  /* The list of groups */
  dialog.list = gtk_clist_new(2);
  gtk_clist_column_titles_hide(GTK_CLIST(dialog.list));
  gtk_clist_set_column_width(GTK_CLIST(dialog.list), 0, 80);
  gtk_widget_set_usize(dialog.list, 400, 200);
  gtk_widget_show(dialog.list);
  gtk_signal_connect(GTK_OBJECT(dialog.list), "select-row",
		     GTK_SIGNAL_FUNC(dialog_select_row), NULL);

  scrolled_window = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window), 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_add(GTK_CONTAINER(scrolled_window), dialog.list);
  gtk_widget_show(scrolled_window);

  gtk_table_attach(GTK_TABLE(table), scrolled_window, 
		   0, 6, 3, 5, 
		   GTK_FILL | GTK_EXPAND, GTK_FILL | GTK_EXPAND, 3, 3);

  /* pack everything up into the dialog */
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog.dialog)->vbox), 
		     table, TRUE, TRUE, 0);
  gtk_widget_show(table);
}

/* this function presets widgets in the dialog according to what is
   held in report_grp_axis_list */
static void dialog_preset_values(void)
{
  GList *cursor;

  for (cursor = report_grp_axis_list; 
       cursor != NULL; 
       cursor = g_list_next(cursor)) {

    ReportGroupData *rgd;

    rgd = g_new(ReportGroupData, 1);
    memcpy(rgd, ReportGroupDataPtr(cursor->data), sizeof(ReportGroupData));

    dialog_setup_rgd(rgd, FALSE);
  }
}

/* dialog_mains syncs the dialog contents with the report_grp_axis_list */
static void dialog_main(void)
{
  gint row;
  
  report_grp_axis_done(); /* free report_grp_axis_list */
  report_grp_axis_list = NULL;

  for (row = 0; row < GTK_CLIST(dialog.list)->rows; row++) {
    ReportGroupData *rgd, *orig;
    orig = ReportGroupDataPtr(
		      gtk_clist_get_row_data(GTK_CLIST(dialog.list), row));
    rgd = g_new(ReportGroupData, 1);
    memcpy(rgd, orig, sizeof(ReportGroupData));

    report_grp_axis_list = g_list_prepend(report_grp_axis_list, rgd);
    gtk_clist_set_row_data(GTK_CLIST(dialog.list), row, NULL);
  }
  report_grp_axis_list = g_list_reverse(report_grp_axis_list);

  if (report_grp_axis_list == NULL) {
    ReportGroupData *rgd = g_new(ReportGroupData, 1);
    strcpy(rgd->label, "");
    strcpy(rgd->expr, "1");
    report_grp_axis_list = g_list_append(report_grp_axis_list, rgd);
  }
}

gboolean report_grp_axis_edit(void)
{
  gint ret;
  
  dialog_build();
  dialog_preset_values();

  ret = gnome_dialog_run(GNOME_DIALOG(dialog.dialog));
  switch (ret) {
  case 0:		/* OK */
    dialog_main();
  case 1:		/* Cancel */
    gnome_dialog_close(GNOME_DIALOG(dialog.dialog));
  case -1:		/* Esc or "closed window" */
    break;
  default:
    g_assert_not_reached();
  }

  return ret == 0; /* Ok */
}

