#include <config.h>

#include "interface.h"

#include "report_progress.h"

struct _Dialog {
  GtkWidget *dialog;
  GtkWidget *progress_bar;

  GTimer *timer;

  gint groups_no;
  gboolean cancel_now;
} dialog;



/* callback function for cancel button */
static void dialog_cancel(GtkWidget *widget, gpointer callback_data)
{
  dialog.cancel_now = TRUE;
}

void report_progress_init(gint groups_no)
{
  dialog.groups_no = groups_no;
  dialog.cancel_now = FALSE;

  dialog.timer = g_timer_new();
  g_timer_start(dialog.timer);

  dialog.progress_bar = gtk_progress_bar_new();
  gtk_progress_set_show_text(GTK_PROGRESS(dialog.progress_bar), TRUE);
  gtk_progress_set_format_string(GTK_PROGRESS(dialog.progress_bar), "%v / %u");
  gtk_progress_configure(GTK_PROGRESS(dialog.progress_bar), 0, 0, groups_no);
  gtk_widget_show(dialog.progress_bar);

  dialog.dialog = gnome_dialog_new(_("Creating report..."), 
				   GNOME_STOCK_BUTTON_CANCEL, NULL);
  gtk_window_set_modal(GTK_WINDOW(dialog.dialog), TRUE);
  gnome_dialog_set_parent(GNOME_DIALOG(dialog.dialog), 
			  GTK_WINDOW(interface.application));
  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog.dialog)->vbox), 
		     dialog.progress_bar, TRUE, TRUE, 0);
  gnome_dialog_button_connect(GNOME_DIALOG(dialog.dialog), 0,
			      GTK_SIGNAL_FUNC(dialog_cancel), NULL);
}

void report_progress_done(void)
{
  gtk_widget_destroy(dialog.dialog);
  g_timer_destroy(dialog.timer);
}

void report_progress_set_group(gint group_no)
{
  gfloat p;
  gulong microseconds;

  gtk_progress_set_value(GTK_PROGRESS(dialog.progress_bar), group_no);

  g_timer_elapsed(dialog.timer, &microseconds);
  p = gtk_progress_get_current_percentage(GTK_PROGRESS(dialog.progress_bar));
  if ((p > 0) && ((microseconds / p) > 2000000))
      gtk_widget_show(dialog.dialog);
}

gboolean report_progress_cancel_now(void)
{
  return dialog.cancel_now;
}



