#include <config.h>

#include <gnome.h>

#include "data.h"
#include "interface.h"
#include "transaction_dlg.h"

#include "misc.h"

#include "schedule.h"


char *frequency_option_text[] = {N_("Daily"),N_("Weekly"),N_("Monthly"),N_("Yearly"), NULL};

void process_scheduled_transactions(void)
{
    ScheduleData *schedule;
    TransactionData *transaction;
    GDate date_today;
    GList *cursor;
    gboolean made_changes = FALSE;
    
    g_date_set_current(&date_today);
    
    for (cursor = schedule_list; cursor != NULL; cursor = g_list_next(cursor)) {
        schedule = SchedulePtr(cursor->data);
        
        while(g_date_compare(&(schedule->date), &date_today) <= 0){
            transaction = g_new(TransactionData, 1);
            
            transaction->amount = schedule->amount;
            transaction->acc_from = schedule->acc_from;
            transaction->acc_to = schedule->acc_to;
            transaction->date = schedule->date;
            strcpy(transaction->desc,schedule->desc);
            transaction->inbound = TRUE;

    	    if(schedule->prompt) {
                 GList *trans_new_list = NULL;

                 trans_new_list = g_list_prepend(trans_new_list, transaction);
     
                 if (transaction_dlg_execute(_("Confirm scheduled transaction ..."), trans_new_list)) {
	             trans_list = g_list_append(trans_list, transaction);
		     interface_build_transaction(transaction);
		     interface_setup_transaction_colors(transaction);
		     made_changes = TRUE;
                 } else
	              g_free(transaction);
     
                g_list_free(trans_new_list);
    		
    	    } else {
		 trans_list = g_list_append(trans_list, transaction);
		 interface_build_transaction(transaction);
		 interface_setup_transaction_colors(transaction);
		 made_changes = TRUE;
	    }
    		
    	    switch(schedule->frequency) {
	    case MONTH:
		 g_date_add_months(&(schedule->date),schedule->factor);
		 break;
	    case WEEK:
		 g_date_add_days(&(schedule->date),7*schedule->factor);
		 break;   	       
	    case DAY:
		 g_date_add_days(&(schedule->date),schedule->factor);
		 break;
	    case YEAR:
		 g_date_add_years(&(schedule->date),schedule->factor);
		 break;
	    }  
    	         	       
	    interface_setup_schedule(SchedulePtr(cursor->data));
        }		   
    }

    if (made_changes) {
	 data_recalculate_accounts();
	 interface_setup_all_accounts_amount();
	 interface_sort_schedules();
	 interface.changed = TRUE;
    }
}
