#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"
#include "acc_entry.h"
#include "date_entry.h"

#include <glib.h>

#include "data.h"
#include "interface.h"

#include "actions.h"
#include "py_module.h"

#include "schedule.h"

#include "schedule_dlg.h"

struct _ScheduleEdit {
     GtkWidget *dialog;

     GtkWidget *from, *to;
     GtkWidget *amount;
     GtkWidget *date;
     GtkWidget *desc;
     GtkWidget *frequency;
     GtkWidget *factor;
     GtkWidget *prompt;

     gint error; /* 0 - ok, 1-7 - number of entry which is wrong */
} schedule_dlg;

/* This really should be in GTK+ (copied from gnome-terminal)
 */
static gint
option_menu_get_history (GtkOptionMenu *option_menu)
{
	GtkWidget *active_widget;
	
	g_return_val_if_fail (GTK_IS_OPTION_MENU (option_menu), -1);
	
	active_widget = gtk_menu_get_active (GTK_MENU (option_menu->menu));

	if (active_widget)
		return g_list_index (GTK_MENU_SHELL (option_menu->menu)->children,
				     active_widget);
	else
		return -1;
}

void schedule_dlg_build_dialog(const gchar *title)
{
    schedule_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(schedule_dlg.dialog), GTK_WINDOW(interface.application));
    gnome_dialog_set_default(GNOME_DIALOG(schedule_dlg.dialog), 0);

    {
        GtkWidget *table;
        GtkWidget *menu;
        GtkWidget *menuitem;
        GSList	  *group;
	gint i;
	
	
	table = gtk_table_new(2, 7, FALSE);

	/* insert entry widgets */
	{
	    schedule_dlg.amount = gtk_entry_new();
	    gtk_widget_show(schedule_dlg.amount);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.amount, 1, 2, 0, 1, GTK_FILL, 0, 3, 3);

	    schedule_dlg.from = acc_entry_new(FALSE);
	    gtk_widget_show(schedule_dlg.from);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.from, 1, 2, 1, 2, GTK_FILL, 0, 3, 3);

	    schedule_dlg.to = acc_entry_new(FALSE);
	    gtk_widget_show(schedule_dlg.to);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.to, 1, 2, 2, 3, GTK_FILL, 0, 3, 3);

	    schedule_dlg.date = date_entry_new();
	    gtk_widget_show(schedule_dlg.date);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.date, 1, 2, 3, 4, GTK_FILL, 0, 3, 3);

	    schedule_dlg.desc = gnome_entry_new("EmmaTransactionEditHistoryId");
	    gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(schedule_dlg.desc)->entry), SCHEDULE_DESC_SIZE-1);
	    gtk_widget_show(schedule_dlg.desc);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.desc, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	    schedule_dlg.frequency = gtk_option_menu_new();
	    menu = gtk_menu_new();
	    group = NULL;
	    i = 0;
	    while (frequency_option_text[i] != NULL){ 
	        menuitem = gtk_menu_item_new_with_label(_(frequency_option_text[i]));
	        i++;
	        gtk_menu_append(GTK_MENU(menu), menuitem);
	        gtk_widget_show(menuitem);
	    }
	    gtk_option_menu_set_menu(GTK_OPTION_MENU(schedule_dlg.frequency), menu);

	    gtk_widget_show(schedule_dlg.frequency);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.frequency, 1, 2, 5, 6, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	    schedule_dlg.factor = gtk_entry_new();
	    gtk_widget_show(schedule_dlg.factor);          
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.factor, 1, 2, 6, 7, GTK_FILL, 0, 3, 3);

	    schedule_dlg.prompt = gtk_check_button_new_with_label (_("Prompt before entering"));
	    gtk_widget_show(schedule_dlg.prompt);
	    gtk_table_attach(GTK_TABLE(table), schedule_dlg.prompt, 1, 2, 7, 8, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	    
	}

	/* insert labels */
	{
	    GtkWidget *label;

	    label = gtk_label_new(_("Amount:"));
	    gtk_widget_show(label);
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("From:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("To:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("First Date:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Description:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, 0, 0, 3, 3);
	    
	    gtk_widget_show(label = gtk_label_new(_("Frequency:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 5, 6, 0, 0, 3, 3);	    

	    gtk_widget_show(label = gtk_label_new(_("Factor:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 6, 7, 0, 0, 3, 3);	    
	}

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(schedule_dlg.dialog)->vbox), table, TRUE, TRUE, 0);
	gtk_widget_show(table);
    }

    /* enter = ok pressed */
    gnome_dialog_editable_enters(GNOME_DIALOG(schedule_dlg.dialog), GTK_EDITABLE(schedule_dlg.amount));
    gnome_dialog_editable_enters(GNOME_DIALOG(schedule_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(schedule_dlg.from)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(schedule_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(schedule_dlg.to)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(schedule_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(schedule_dlg.date)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(schedule_dlg.dialog), 
				 GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc))));
}

void schedule_dlg_preset_values(GList *schedule_selection)
{
     GList *cursor;
     ScheduleData *schedule, *schedule2;
     int amount_diff = FALSE, from_diff   = FALSE, to_diff        = FALSE, 
         date_diff   = FALSE, desc_diff   = FALSE, frequency_diff = FALSE, 
         factor_diff = FALSE, prompt_diff = FALSE;

     for (cursor = schedule_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	  if (cursor->prev == NULL)
	       continue;

	  schedule = SchedulePtr(cursor->data);
	  schedule2 = SchedulePtr(cursor->prev->data);

	  amount_diff |= (schedule->amount != schedule2->amount);
	  from_diff |= (schedule->acc_from != schedule2->acc_from);
	  to_diff |= (schedule->acc_to != schedule2->acc_to);
	  date_diff |= g_date_compare(&(schedule->date), &(schedule2->date));
	  desc_diff |= strcmp(schedule->desc, schedule2->desc);
	  frequency_diff |= (schedule->frequency != schedule2->frequency);
    factor_diff |= (schedule->factor != schedule2->factor);
	  prompt_diff |= (schedule->prompt != schedule2->prompt);
     }

     schedule = SchedulePtr(schedule_selection->data);

     if (amount_diff)
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.amount), _("<don't change>"));
     else {
	  char text[16];
	  sprintf(text, "%.2f", schedule->amount);
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.amount), text);
     }

     if (from_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), _("<don't change>"));
     else 
	  if (schedule->acc_from == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry), schedule->acc_from->name);
     
     if (to_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), _("<don't change>"));
     else 
	  if (schedule->acc_to == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry), schedule->acc_to->name);

     if (date_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.date)->entry), _("<don't change>"));
     else
	  date_entry_set_date(schedule_dlg.date, &(schedule->date));

     if (desc_diff)
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc))), _("<don't change>"));
     else
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc))), schedule->desc);
 
    gtk_option_menu_set_history(GTK_OPTION_MENU(schedule_dlg.frequency), schedule->frequency);
    
     if (factor_diff)
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.factor), _("<don't change>"));
     else {
	  char text[16];
	  sprintf(text, "%d", schedule->factor);
	  gtk_entry_set_text(GTK_ENTRY(schedule_dlg.factor), text);
     }


    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(schedule_dlg.prompt), schedule->prompt ? 1 : 0);
}

void schedule_dlg_handle_errors(void)
{
     switch (schedule_dlg.error) {
     case 1: /* amount */
	  gtk_widget_grab_focus(schedule_dlg.amount);

	  if (py_module_error_occurred()) {
	       action_error_msg(_("Error in amount expression!\n\nPython says:\n"),
				py_module_get_error_string());
	       py_module_clear_error();
	  } else 
	       gdk_beep();

	  break;
     case 2: /* from */
	  gtk_widget_grab_focus(schedule_dlg.from);
	  gdk_beep();
	  break;
     case 3: /* to */
	  gtk_widget_grab_focus(schedule_dlg.to);
	  gdk_beep();
	  break;
     case 4: /* date */
	  gtk_widget_grab_focus(schedule_dlg.date);
	  gdk_beep();
	  break;
     case 5: /* desc */
	  gtk_widget_grab_focus(schedule_dlg.desc);
	  gdk_beep();
	  break;
     default:
	  g_assert_not_reached();
     }
}

/* returns 0 on success or number of field which coudn't be read */
int schedule_dlg_fetch_values(gdouble **amount_val, AccountData **from_val, AccountData **to_val,
			      GDate **date_val,    gchar    **desc_val,  gint **frequency_val, 
			      gint  ** factor_val, gboolean **prompt_val)
{
     /* amount */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(schedule_dlg.amount)), _("<don't change>")) == 0) {
	  g_free(*amount_val);
	  *amount_val = NULL;
     } else {
	  gchar *expr;
	  expr = gtk_entry_get_text(GTK_ENTRY(schedule_dlg.amount));
	  **amount_val = py_module_eval_amount(expr);
	  if (py_module_error_occurred())
	       return 1;
     }

     /* from and to */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.from)->entry)), _("<don't change>")) == 0)
	  *from_val = NULL;
     else {
	  if (acc_entry_get_account(schedule_dlg.from, from_val) || (*from_val == NULL))
	       return 2;
     }

     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.to)->entry)), _("<don't change>")) == 0)
	  *to_val = NULL;
     else {
	  if (acc_entry_get_account(schedule_dlg.to, to_val) || (*to_val == NULL))
	       return 3;
     }

     /* date */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(schedule_dlg.date)->entry)), _("<don't change>")) == 0) {
	  g_date_free(*date_val);
	  *date_val = NULL;
     } else {
	  if (date_entry_get_date(schedule_dlg.date, *date_val))
	       return 4;
     }

     /* desc */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(schedule_dlg.desc)))),
		_("<don't change>")) == 0)
	  *desc_val = NULL;
     else {
	  *desc_val = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(schedule_dlg.desc)->entry));
	  *desc_val = g_strstrip(*desc_val);
     }

     /* frequency and prompt */
     **frequency_val = option_menu_get_history(GTK_OPTION_MENU(schedule_dlg.frequency));
     **prompt_val = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(schedule_dlg.prompt));

     /* amount */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(schedule_dlg.factor)), _("<don't change>")) == 0) {
	  g_free(*factor_val);
	  *factor_val = NULL;
     } else {
	  gchar *expr;
	  expr = gtk_entry_get_text(GTK_ENTRY(schedule_dlg.factor));
	  **factor_val = py_module_eval_amount(expr);
	  if (py_module_error_occurred())
	       return 1;
     }
     
     return 0;
}

void schedule_dlg_main(GList * schedule_selection)
{
    GList *cursor;

    gdouble *amount_val;
    AccountData *from_val, *to_val;
    GDate *date_val;
    gchar *description_val;
    gint *frequency_val;
    gint *factor_val;
    gboolean *prompt_val;

    amount_val = g_new(gdouble, 1);
    date_val = g_date_new();
    frequency_val = g_new(gint, 1);
    factor_val = g_new(gint,1);
    prompt_val = g_new(gboolean, 1);

    schedule_dlg.error = schedule_dlg_fetch_values(&amount_val, &from_val, &to_val, &date_val,
						   &description_val, &frequency_val, &factor_val, &prompt_val);
    if (schedule_dlg.error != 0)
	 goto err_exit;

    /* and now we can actually change schedules on the list */
    for (cursor = schedule_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	ScheduleData *schedule = SchedulePtr(cursor->data);

	if (amount_val != NULL)
	    schedule->amount = *amount_val;
	if (from_val != NULL)
	    schedule->acc_from = from_val;
	if (to_val != NULL)
	    schedule->acc_to = to_val;
	if (date_val != NULL)
	     schedule->date = *date_val;
	if (description_val != NULL) 
	     strcpy(schedule->desc, description_val);
	if (frequency_val != NULL)
	     schedule->frequency = *frequency_val;
	if (factor_val != NULL)
	     schedule->factor = *factor_val;
	if (prompt_val != NULL)
	     schedule->prompt = *prompt_val;
    }

 err_exit:
    g_free(amount_val);
    g_date_free(date_val);
    g_free(frequency_val);
    g_free(prompt_val);
    g_free(factor_val);
}

gboolean schedule_dlg_execute(const gchar *title, GList * schedule_selection)
{
    schedule_dlg_build_dialog(title);
    schedule_dlg_preset_values(schedule_selection);

    while (TRUE) {
	int ret;

	ret = gnome_dialog_run(GNOME_DIALOG(schedule_dlg.dialog));
	switch (ret) {
	case 0:		/* OK */
	    schedule_dlg_main(schedule_selection);
	    if (schedule_dlg.error == 0) {
		gnome_dialog_close(GNOME_DIALOG(schedule_dlg.dialog));
		return TRUE;
	    } else
		 schedule_dlg_handle_errors();

	    break;
	case 1:		/* Cancel */
	    gnome_dialog_close(GNOME_DIALOG(schedule_dlg.dialog));
	case -1:		/* Esc or "closed window" */
	    return FALSE;
	}
    };
}

