#include <config.h>

#include "actions.h"
#include "data.h"
#include "interface.h"

#include "sm.h"

static gint 
sm_save_yourself (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
		  gint is_shutdown, GnomeInteractStyle interact_style,
		  gint is_fast, gpointer client_data)
{
     gchar *discard_argv[5] = { "rm", "-r", NULL, NULL, NULL };
     gint discard_argc = 2;
     gchar *config_file = NULL, *tmp_file = NULL;

     gnome_config_push_prefix (gnome_client_get_config_prefix (client));

     config_file = g_strdup(gnome_config_get_real_path(gnome_client_get_config_prefix (client)));
     if (config_file[strlen(config_file)-1] == G_DIR_SEPARATOR) 
	  config_file[strlen(config_file)-1] = '\0';
     discard_argv[discard_argc++] = config_file;

     if ((save_style == GNOME_SAVE_LOCAL) || (strcmp(interface.filename, "") == 0)) {
	  tmp_file = g_strdup_printf("%s%s", config_file, ".data");
		    
	  gnome_config_set_string("Savefile/TmpFilename", tmp_file);
	  gnome_config_set_string("Savefile/RealFilename", interface.filename);
     
	  data_write(tmp_file);
	  /* FIXME - check for errors */
	  
	  discard_argv[discard_argc++] = tmp_file;
     } else {
	  gnome_config_set_string("Savefile/TmpFilename", interface.filename);
	  gnome_config_set_string("Savefile/RealFilename", interface.filename);
     
	  if (interface.changed) {
	       data_write(interface.filename);
	       /* FIXME - check for errors */
	  }
     }

     if ((save_style == GNOME_SAVE_LOCAL) || (save_style == GNOME_SAVE_BOTH)) {
	  gnome_config_set_bool("State/Saved", TRUE);

	  gnome_config_set_string("State/Limit", interface_get_limit());

	  /* FIXME - save other widget's state ? */
     }

     gnome_config_pop_prefix ();
     gnome_config_sync();
	  
     gnome_client_set_discard_command (client, discard_argc, discard_argv);

     g_free(tmp_file);

     return TRUE;
}

void sm_reload_limit(void)
{
    if (gnome_config_get_bool("State/Saved=False")) {
	  gchar *text;

	  text = gnome_config_get_string("State/Limit");
	  interface_setup_limit(text);
     }
}

void sm_reload_datafile(void)
{
     gchar *tmp_filename, *real_filename;

     tmp_filename = gnome_config_get_string("Savefile/TmpFilename");
     real_filename = gnome_config_get_string("Savefile/RealFilename");

     action_open_direct(tmp_filename);
     interface_setup_filename(real_filename);
     interface.changed = (strcmp(tmp_filename, real_filename) != 0);

     g_free(tmp_filename);
     g_free(real_filename);
}

void sm_reload_widgets_state(void)
{
     /* FIXME - restore other widget's state ? */
}

void sm_restart(GnomeClient *client)
{
     gnome_config_push_prefix (gnome_client_get_config_prefix (client));

     sm_reload_limit();
     sm_reload_datafile();
     sm_reload_widgets_state();

     gnome_config_pop_prefix ();
}

void sm_init(void)
{
     GnomeClient* client;

     client = gnome_master_client ();
     gtk_signal_connect (GTK_OBJECT (client), "save_yourself",
			 GTK_SIGNAL_FUNC (sm_save_yourself), NULL);
     gtk_signal_connect (GTK_OBJECT (client), "die",
			 GTK_SIGNAL_FUNC (gtk_main_quit), NULL);

     if (gnome_client_get_flags(GNOME_CLIENT(client)) & (GNOME_CLIENT_RESTARTED | GNOME_CLIENT_RESTORED))
	  sm_restart(client);
}

