#include <config.h>

#include <gnome.h>
#include "gnome-ucombo.h"
#include "acc_entry.h"
#include "date_entry.h"

#include <glib.h>

#include "data.h"
#include "interface.h"

#include "actions.h"
#include "py_module.h"

#include "transaction_dlg.h"


struct _TransactionEdit {
     GtkWidget *dialog;

     GtkWidget *from, *to;
     GtkWidget *amount;
     GtkWidget *date;
     GtkWidget *desc;

     gint error; /* 0 - ok, 1-5 - number of entry which is wrong */
} transaction_dlg;

void transaction_dlg_build_dialog(const gchar *title)
{
    transaction_dlg.dialog = gnome_dialog_new(title, GNOME_STOCK_BUTTON_OK, GNOME_STOCK_BUTTON_CANCEL, NULL);
    gnome_dialog_set_parent(GNOME_DIALOG(transaction_dlg.dialog), GTK_WINDOW(interface.application));
    gnome_dialog_set_default(GNOME_DIALOG(transaction_dlg.dialog), 0);

    {
	GtkWidget *table;
	table = gtk_table_new(2, 5, FALSE);

	/* insert entry widgets */
	{
	    transaction_dlg.amount = gtk_entry_new();
	    gtk_widget_show(transaction_dlg.amount);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.amount, 1, 2, 0, 1, GTK_FILL, 0, 3, 3);

	    transaction_dlg.from = acc_entry_new(FALSE);
	    gtk_widget_show(transaction_dlg.from);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.from, 1, 2, 1, 2, GTK_FILL, 0, 3, 3);

	    transaction_dlg.to = acc_entry_new(FALSE);
	    gtk_widget_show(transaction_dlg.to);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.to, 1, 2, 2, 3, GTK_FILL, 0, 3, 3);

	    transaction_dlg.date = date_entry_new();
	    gtk_widget_show(transaction_dlg.date);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.date, 1, 2, 3, 4, GTK_FILL, 0, 3, 3);

	    transaction_dlg.desc = gnome_entry_new("EmmaTransactionEditHistoryId");
	    gtk_entry_set_max_length(GTK_ENTRY(GTK_COMBO(transaction_dlg.desc)->entry), TRANSACTION_DESC_SIZE-1);
	    gtk_widget_show(transaction_dlg.desc);
	    gtk_table_attach(GTK_TABLE(table), transaction_dlg.desc, 1, 2, 4, 5, GTK_FILL | GTK_EXPAND, 0, 3, 3);
	}

	/* insert labels */
	{
	    GtkWidget *label;

	    label = gtk_label_new(_("Amount:"));
	    gtk_widget_show(label);
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1, GTK_FILL, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("From:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("To:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Date:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4, 0, 0, 3, 3);

	    gtk_widget_show(label = gtk_label_new(_("Description:")));
	    gtk_table_attach(GTK_TABLE(table), label, 0, 1, 4, 5, 0, 0, 3, 3);
	}

	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(transaction_dlg.dialog)->vbox), table, TRUE, TRUE, 0);
	gtk_widget_show(table);
    }

    /* enter = ok pressed */
    gnome_dialog_editable_enters(GNOME_DIALOG(transaction_dlg.dialog), GTK_EDITABLE(transaction_dlg.amount));
    gnome_dialog_editable_enters(GNOME_DIALOG(transaction_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(transaction_dlg.from)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(transaction_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(transaction_dlg.to)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(transaction_dlg.dialog), 
				 GTK_EDITABLE(GNOME_UCOMBO(transaction_dlg.date)->entry));
    gnome_dialog_editable_enters(GNOME_DIALOG(transaction_dlg.dialog), 
				 GTK_EDITABLE(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc))));
}

void transaction_dlg_preset_values(GList *trans_selection)
{
     GList *cursor;
     TransactionData *trans, *trans2;
     int amount_diff = FALSE, from_diff = FALSE, to_diff = FALSE, date_diff = FALSE, desc_diff = FALSE;

     for (cursor = trans_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	  if (cursor->prev == NULL)
	       continue;

	  trans = TransactionPtr(cursor->data);
	  trans2 = TransactionPtr(cursor->prev->data);

	  amount_diff |= (trans->amount != trans2->amount);
	  from_diff |= (trans->acc_from != trans2->acc_from);
	  to_diff |= (trans->acc_to != trans2->acc_to);
	  date_diff |= g_date_compare(&(trans->date), &(trans2->date));
	  desc_diff |= strcmp(trans->desc, trans2->desc);
     }

     trans = TransactionPtr(trans_selection->data);

     if (amount_diff)
	  gtk_entry_set_text(GTK_ENTRY(transaction_dlg.amount), _("<don't change>"));
     else {
	  char text[16];
	  sprintf(text, "%.2f", trans->amount);
	  gtk_entry_set_text(GTK_ENTRY(transaction_dlg.amount), text);
     }

     if (from_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), _("<don't change>"));
     else 
	  if (trans->acc_from == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry), trans->acc_from->name);
     
     if (to_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), _("<don't change>"));
     else 
	  if (trans->acc_to == NULL)
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), "");
	  else
	       gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry), trans->acc_to->name);

     if (date_diff)
	  gtk_entry_set_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.date)->entry), _("<don't change>"));
     else
	  date_entry_set_date(transaction_dlg.date, &(trans->date));

     if (desc_diff)
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc))), _("<don't change>"));
     else
	  gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc))), trans->desc);

}

void transaction_dlg_handle_errors(void)
{
     switch (transaction_dlg.error) {
     case 1: /* amount */
	  gtk_widget_grab_focus(transaction_dlg.amount);

	  if (py_module_error_occurred()) {
	       action_error_msg(_("Error in amount expression!\n\nPython says:\n"),
				py_module_get_error_string());
	       py_module_clear_error();
	  } else 
	       gdk_beep();

	  break;
     case 2: /* from */
	  gtk_widget_grab_focus(transaction_dlg.from);
	  gdk_beep();
	  break;
     case 3: /* to */
	  gtk_widget_grab_focus(transaction_dlg.to);
	  gdk_beep();
	  break;
     case 4: /* date */
	  gtk_widget_grab_focus(transaction_dlg.date);
	  gdk_beep();
	  break;
     case 5: /* desc */
	  gtk_widget_grab_focus(transaction_dlg.desc);
	  gdk_beep();
	  break;
     default:
	  g_assert_not_reached();
     }
}

/* returns 0 on success or number of field which coudn't be read */
int transaction_dlg_fetch_values(gdouble **amount_val, AccountData **from_val, AccountData **to_val,
				 GDate **date_val, gchar **desc_val)
{
     /* amount */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(transaction_dlg.amount)), _("<don't change>")) == 0) {
	  g_free(*amount_val);
	  *amount_val = NULL;
     } else {
	  gchar *expr;
	  expr = gtk_entry_get_text(GTK_ENTRY(transaction_dlg.amount));
	  **amount_val = py_module_eval_amount(expr);
	  if (py_module_error_occurred())
	       return 1;
     }

     /* from and to */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.from)->entry)), _("<don't change>")) == 0)
	  *from_val = NULL;
     else {
	  if (acc_entry_get_account(transaction_dlg.from, from_val) || (*from_val == NULL))
	       return 2;
     }

     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.to)->entry)), _("<don't change>")) == 0)
	  *to_val = NULL;
     else {
	  if (acc_entry_get_account(transaction_dlg.to, to_val) || (*to_val == NULL))
	       return 3;
     }

     /* date */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(GNOME_UCOMBO(transaction_dlg.date)->entry)), _("<don't change>")) == 0) {
	  g_date_free(*date_val);
	  *date_val = NULL;
     } else {
	  if (date_entry_get_date(transaction_dlg.date, *date_val))
	       return 4;
     }

     /* desc */
     if (strcmp(gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(transaction_dlg.desc)))),
		_("<don't change>")) == 0)
	  *desc_val = NULL;
     else {
	  *desc_val = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(transaction_dlg.desc)->entry));
	  *desc_val = g_strstrip(*desc_val);
     }

     return 0;
}

void transaction_dlg_main(GList *trans_selection)
{
    GList *cursor;
 
    gdouble *amount_val;
    AccountData *from_val, *to_val;
    GDate *date_val;
    gchar *description_val;

    amount_val = g_new(gdouble, 1);
    date_val = g_date_new();

    transaction_dlg.error = transaction_dlg_fetch_values(&amount_val, &from_val, &to_val, 
							 &date_val, &description_val);
    if (transaction_dlg.error != 0)
	 goto err_exit;

    /* and now we can actually change transactions on the list */
    for (cursor = trans_selection; cursor != NULL; cursor = g_list_next(cursor)) {
	TransactionData *transaction = TransactionPtr(cursor->data);

	if (amount_val != NULL)
	    transaction->amount = *amount_val;
	if (from_val != NULL)
	    transaction->acc_from = from_val;
	if (to_val != NULL)
	    transaction->acc_to = to_val;
	if (date_val != NULL)
	     transaction->date = *date_val;
	if (description_val != NULL) 
	     strcpy(transaction->desc, description_val);
    }

 err_exit:
    g_free(amount_val);
    g_date_free(date_val);
}

gboolean transaction_dlg_execute(const gchar *title, GList * trans_selection)
{
    transaction_dlg_build_dialog(title);
    transaction_dlg_preset_values(trans_selection);

    while (TRUE) {
	int ret;

	ret = gnome_dialog_run(GNOME_DIALOG(transaction_dlg.dialog));
	switch (ret) {
	case 0:		/* OK */
	     transaction_dlg_main(trans_selection);
	     if (transaction_dlg.error == 0) {
		  gnome_dialog_close(GNOME_DIALOG(transaction_dlg.dialog));
		  return TRUE;
	     } else
		  transaction_dlg_handle_errors();

	     break;
	case 1:		/* Cancel */
	     gnome_dialog_close(GNOME_DIALOG(transaction_dlg.dialog));
	case -1:		/* Esc or "closed window" */
	     return FALSE;
	}
    };
}
